import os
import textwrap
import unittest
from collections import OrderedDict

from parameterized import parameterized

from conans.paths import CONANFILE
from conans.test.utils.tools import NO_SETTINGS_PACKAGE_ID, TestClient, TestServer, \
    inc_package_manifest_timestamp, inc_recipe_manifest_timestamp
from conans.util.files import load


class VersionRangesUpdatingTest(unittest.TestCase):

    def update_remote_test(self):
        # https://github.com/conan-io/conan/issues/5333
        client = TestClient(servers={"default": TestServer()},
                            users={"default": [("lasote", "mypass")]})
        conanfile = textwrap.dedent("""
            from conans import ConanFile
            class Boost(ConanFile):
                pass
            """)
        client.save({"conanfile.py": conanfile})
        client.run("create . boost/1.68.0@lasote/stable")
        client.run("create . boost/1.69.0@lasote/stable")
        client.run("create . boost/1.70.0@lasote/stable")
        client.run("upload * -r=default --all --confirm")
        client.run("remove * -f")
        conanfile = textwrap.dedent("""
            [requires]
            boost/[>=1.68.0]@lasote/stable
            """)
        client.save({"conanfile.txt": conanfile}, clean_first=True)
        client.run("install .")   
        self.assertIn("boost/*@lasote/stable versions found in 'default' remote", client.out)
        self.assertIn("resolved to 'boost/1.70.0@lasote/stable' in remote 'default'", client.out)
        self.assertNotIn("boost/1.69.0", client.out)
        self.assertNotIn("boost/1.68.0", client.out)
        client.run("install .")
        self.assertIn("resolved to 'boost/1.70.0@lasote/stable' in local cache", client.out)
        self.assertIn("boost/1.70.0", client.out)
        self.assertNotIn("boost/1.69.0", client.out)
        self.assertNotIn("boost/1.68.0", client.out)

        client.run("install . --update")
        self.assertIn("resolved to 'boost/1.70.0@lasote/stable' in remote 'default'", client.out)
        self.assertIn("boost/1.70.0", client.out)
        self.assertNotIn("boost/1.69.0", client.out)
        self.assertNotIn("boost/1.68.0", client.out)

    def update_test(self):
        client = TestClient(servers={"default": TestServer()},
                            users={"default": [("lasote", "mypass")]})
        conanfile = """from conans import ConanFile
class HelloReuseConan(ConanFile):
    pass
"""
        client.save({"conanfile.py": conanfile})
        client.run("create . Pkg/1.1@lasote/testing")
        client.run("create . Pkg/1.2@lasote/testing")
        client.run("upload Pkg* -r=default --all --confirm")
        client.run("remove Pkg/1.2@lasote/testing -f")
        conanfile = """from conans import ConanFile
class HelloReuseConan(ConanFile):
    requires = "Pkg/[~1]@lasote/testing"
"""
        client.save({"conanfile.py": conanfile})
        client.run("install .")
        # Resolves to local package
        self.assertIn("Pkg/1.1@lasote/testing: Already installed!", client.out)
        client.run("install . --update")
        # Resolves to remote package
        self.assertIn("Pkg/1.2@lasote/testing: Package installed", client.out)
        self.assertNotIn("Pkg/1.1", client.out)

        # removes remote
        client.run("remove Pkg* -r=default --f")
        # Resolves to local package
        client.run("install .")
        self.assertIn("Pkg/1.2@lasote/testing: Already installed!", client.out)
        # Update also resolves to local package
        client.run("install . --update")
        self.assertIn("Pkg/1.2@lasote/testing: Already installed!", client.out)
        self.assertNotIn("Pkg/1.1", client.out)

    def update_pkg_test(self):
        server = TestServer()
        client = TestClient(servers={"default": server},
                            users={"default": [("lasote", "mypass")]})
        conanfile = """from conans import ConanFile
class HelloReuseConan(ConanFile):
    def package_info(self):
        self.output.info("PACKAGE_INFO {}")
"""
        client.save({"conanfile.py": conanfile.format("1.1")})
        client.run("create . Pkg/1.1@lasote/testing")
        client.save({"conanfile.py": conanfile.format("1.2")})
        client.run("create . Pkg/1.2@lasote/testing")
        client.run("upload Pkg* -r=default --all --confirm")
        consumer = """from conans import ConanFile
class HelloReuseConan(ConanFile):
    requires = "Pkg/[~1]@lasote/testing"
"""
        client.save({"conanfile.py": consumer})
        client.run("install .")
        # Resolves to local package
        self.assertIn("Pkg/1.2@lasote/testing: Already installed!", client.out)
        self.assertIn("Pkg/1.2@lasote/testing: PACKAGE_INFO 1.2", client.out)

        # modify remote 1.2
        client2 = TestClient(servers={"default": server},
                             users={"default": [("lasote", "mypass")]})
        client2.save({"conanfile.py": conanfile.format("*1.2*")})
        client2.run("create . Pkg/1.2@lasote/testing")

        # Make sure timestamp increases, in some machines in testing,
        # it can fail due to same timestamp
        inc_recipe_manifest_timestamp(client2.cache, "Pkg/1.2@lasote/testing", 1)
        inc_package_manifest_timestamp(client2.cache,
                                       "Pkg/1.2@lasote/testing:%s" % NO_SETTINGS_PACKAGE_ID,
                                       1)

        client2.run("upload Pkg* -r=default --all --confirm")

        client.run("install .")
        # Resolves to local package
        self.assertIn("Pkg/1.2@lasote/testing: Already installed!", client.out)
        self.assertIn("Pkg/1.2@lasote/testing: PACKAGE_INFO 1.2", client.out)

        client.run("install . --update")
        # Resolves to remote new recipe and package
        self.assertIn("Pkg/1.2@lasote/testing: Package installed", client.out)
        self.assertNotIn("Pkg/1.2@lasote/testing: PACKAGE_INFO 1.2", client.out)
        self.assertIn("Pkg/1.2@lasote/testing: PACKAGE_INFO *1.2*", client.out)


class VersionRangesMultiRemoteTest(unittest.TestCase):

    def setUp(self):
        self.servers = OrderedDict()
        self.servers["default"] = TestServer()
        self.servers["other"] = TestServer()
        self.client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")],
                                                              "other": [("lasote", "mypass")]})

    def _export(self, name, version, deps=None, export=True, upload=True, remote="default"):
        deps = ", ".join(['"%s"' % d for d in deps or []]) or '""'
        conanfile = """
from conans import ConanFile, CMake
import os

class HelloReuseConan(ConanFile):
    name = "%s"
    version = "%s"
    requires = %s
""" % (name, version, deps)
        files = {CONANFILE: conanfile}
        self.client.save(files, clean_first=True)
        if export:
            self.client.run("export . lasote/stable")
            if upload:
                self.client.run("upload %s/%s@lasote/stable -r=%s" % (name, version, remote))

    def resolve_from_remotes_test(self):
        self._export("Hello0", "0.1")
        self._export("Hello0", "0.2")
        self._export("Hello0", "0.3", remote="other")
        self._export("Hello1", "0.1", ["Hello0/[>0.1,<0.4]@lasote/stable"], export=False,
                     upload=False)

        for remote, solution in [("default", "0.2"), ("other", "0.3")]:
            self.client.run('remove "Hello0/0.*" -f')
            self.client.run("install . --build missing -r=%s" % remote)
            self.assertIn("Version range '>0.1,<0.4' required by "
                          "'conanfile.py (Hello1/0.1)' "
                          "resolved to 'Hello0/%s@lasote/stable'" % solution,
                          self.client.out)
            self.assertIn("conanfile.py (Hello1/0.1): Generated conaninfo.txt",
                          self.client.out)
            content = load(os.path.join(self.client.current_folder, "conaninfo.txt"))
            self.assertIn("Hello0/%s@lasote/stable" % solution, content)


class VersionRangesDiamondTest(unittest.TestCase):

    def setUp(self):
        test_server = TestServer()
        self.servers = {"default": test_server}
        self.client = TestClient(servers=self.servers, users={"default": [("lasote", "mypass")]})

    def _export(self, name, version, deps=None, export=True, upload=True):
        deps = ", ".join(['"%s"' % d for d in deps or []]) or '""'
        conanfile = """
from conans import ConanFile, CMake
import os

class HelloReuseConan(ConanFile):
    name = "%s"
    version = "%s"
    requires = %s
""" % (name, version, deps)
        files = {CONANFILE: conanfile}
        self.client.save(files, clean_first=True)
        if export:
            self.client.run("export . lasote/stable")
            if upload:
                self.client.run("upload %s/%s@lasote/stable" % (name, version))

    def local_then_remote_test(self):
        self._export("Hello0", "0.1")
        self._export("Hello0", "0.2")
        self._export("Hello0", "0.3")
        self._export("Hello0", "1.4")
        self._export("Hello1", "0.1", ["Hello0/[>0.1,<0.3]@lasote/stable"], export=False,
                     upload=False)

        self.client.run('remove "Hello0/0.*" -f')
        self.client.run("install . --build missing")
        self.assertIn("Version range '>0.1,<0.3' required by 'conanfile.py (Hello1/0.1)' "
                      "resolved to 'Hello0/0.2@lasote/stable'", self.client.out)
        self.assertIn("conanfile.py (Hello1/0.1): Generated conaninfo.txt",
                      self.client.out)

        content = load(os.path.join(self.client.current_folder, "conaninfo.txt"))
        self.assertIn("Hello0/0.2@lasote/stable", content)

    @parameterized.expand([(False, ), (True,)])
    def reuse_test(self, upload):
        self._export("Hello0", "0.1", upload=upload)
        self._export("Hello0", "0.2", upload=upload)
        self._export("Hello0", "0.3", upload=upload)
        self._export("Hello1", "0.1", ["Hello0/[>0.1,<0.3]@lasote/stable"], upload=upload)
        self._export("Hello2", "0.1", ["Hello0/[0.2]@lasote/stable"], upload=upload)
        self._export("Hello3", "0.1", ["Hello1/[>=0]@lasote/stable", "Hello2/[~=0]@lasote/stable"],
                     export=False, upload=upload)

        if upload:
            self.client.run('remove "*" -f')

        self.client.run("install . --build missing")

        def check1():
            self.assertIn("Version range '~=0' required by 'conanfile.py (Hello3/0.1)' "
                          "resolved to 'Hello2/0.1@lasote/stable'", self.client.out)
            self.assertIn("Version range '>0.1,<0.3' required by 'Hello1/0.1@lasote/stable' "
                          "resolved to 'Hello0/0.2@lasote/stable'", self.client.out)
            self.assertIn("Version range '0.2' required by 'Hello2/0.1@lasote/stable' resolved "
                          "to 'Hello0/0.2@lasote/stable'", self.client.out)
            self.assertNotIn("Conflict", self.client.out)
            self.assertIn("conanfile.py (Hello3/0.1): Generated conaninfo.txt",
                          self.client.out)

            content = load(os.path.join(self.client.current_folder, "conaninfo.txt"))
            self.assertIn("Hello0/0.2@lasote/stable", content)
            self.assertIn("Hello1/0.1@lasote/stable", content)
            self.assertIn("Hello2/0.1@lasote/stable", content)

        check1()

        if upload:
            self._export("Hello0", "0.2.1", upload=upload)
            self.client.run('remove Hello0/0.2.1@lasote/stable -f')
            self._export("Hello3", "0.1", ["Hello1/[>=0]@lasote/stable",
                                           "Hello2/[~=0]@lasote/stable"],
                         export=False, upload=upload)
            self.client.run("install . --build missing")
            check1()
            # Now update
            self.client.run("install . --update --build missing")
            self.assertIn("Version range '~=0' required by 'conanfile.py (Hello3/0.1)' "
                          "resolved to 'Hello2/0.1@lasote/stable'", self.client.out)
            self.assertIn("Version range '>0.1,<0.3' required by 'Hello1/0.1@lasote/stable' "
                          "resolved to 'Hello0/0.2.1@lasote/stable'", self.client.out)
            self.assertIn("Version range '0.2' required by 'Hello2/0.1@lasote/stable' resolved "
                          "to 'Hello0/0.2.1@lasote/stable'", self.client.out)
            self.assertNotIn("Conflict", self.client.out)
            self.assertIn("conanfile.py (Hello3/0.1): Generated conaninfo.txt",
                          self.client.out)

            content = load(os.path.join(self.client.current_folder, "conaninfo.txt"))
            self.assertIn("Hello0/0.2.1@lasote/stable", content)
            self.assertIn("Hello1/0.1@lasote/stable", content)
            self.assertIn("Hello2/0.1@lasote/stable", content)

    def no_joint_compatibility_resolved_test(self):
        """Test to verify that conan is not resolving using joint-compatibility of the full graph
        and you need to specify the right order or override downstream the conflict"""
        self._export("ProblemRequirement", "1.0.0", upload=True)
        self._export("ProblemRequirement", "1.1.0", upload=True)
        self._export("RequirementOne", "1.2.3",
                     ["ProblemRequirement/[=1.0.0]@lasote/stable"], upload=True)
        self._export("RequirementTwo", "4.5.6",
                     ["ProblemRequirement/[~1]@lasote/stable"], upload=True)
        self._export("Project", "1.0.0",
                     ["RequirementTwo/[=4.5.6]@lasote/stable",
                      "RequirementOne/[=1.2.3]@lasote/stable"], upload=True)

        self.client.run("remove '*' -f")
        self.client.run("install Project/1.0.0@lasote/stable --build missing", assert_error=True)

        self.assertIn("Requirement ProblemRequirement/1.0.0@lasote/stable conflicts with "
                      "already defined ProblemRequirement/1.1.0@lasote/stable", self.client.out)

        # Change the order, now it resolves correctly
        self._export("Project", "1.0.0",
                     ["RequirementOne/[=1.2.3]@lasote/stable",
                      "RequirementTwo/[=4.5.6]@lasote/stable",
                      ], upload=True)
        self.client.run("remove '*' -f")
        self.client.run("install Project/1.0.0@lasote/stable --build missing")



import os
import unittest

from conans.server.plugin_loader import load_authentication_plugin
from conans.test.utils.test_files import temp_folder
from conans.util.files import save


class AuthenticatorPluginTest(unittest.TestCase):

    def instance_authenticator_test(self):
        folder = temp_folder()
        plugin_path = os.path.join(folder, "plugins", "authenticator", "my_auth.py")
        my_plugin = '''
# import to test that they work
import os

def get_class():
    return MyAuthenticator()


class MyAuthenticator(object):
    def valid_user(self, username, plain_password):
        os.path.exists("somepath")  # dummy call, to test that os is not removed by GC
        return username == "foo" and plain_password == "bar"
'''
        save(plugin_path, my_plugin)

        plugin = load_authentication_plugin(folder, "my_auth")
        self.assertTrue(plugin.valid_user("foo", "bar"))
        self.assertFalse(plugin.valid_user("foo2", "bar2"))



# coding=utf-8

import os
import unittest

from conans.client.tools.files import save, chdir
from conans.client.tools import apple_dot_clean
from conans.test.utils.test_files import temp_folder


class DotCleanTest(unittest.TestCase):

    def _run_test(self, tuples):
        tmp_folder = temp_folder()
        with chdir(tmp_folder):
            for f, _ in tuples:
                save(f, "")

            apple_dot_clean(".")

            for f, expected_after in tuples:
                self.assertEqual(expected_after, os.path.exists(f))

    def test_removal_normal(self):
        files = [("file.txt", True),
                 ("._file.txt", False),
                 ("folder/file.txt", True),
                 ("folder/._file.txt", False),]
        self._run_test(tuples=files)

    def test_only_remove_matching_ones(self):
        files = [("file.txt", True),
                 ("._file.txt", False),
                 ("._other.txt", True)]
        self._run_test(tuples=files)

    def test_handle_dirs(self):
        files = [("folder/file.txt", True),
                 ("folder/._file.txt", False),
                 ("._folder/file.txt", True),
                 ("._folder/._file.txt", False),
                 ("._other/._file.txt", True),
                 ("._other2/file.txt", True)]
        self._run_test(tuples=files)



import os
import shutil
import time
import traceback

from requests.exceptions import ConnectionError

from conans import DEFAULT_REVISION_V1
from conans.client.cache.remote_registry import Remote
from conans.client.source import merge_directories
from conans.errors import ConanConnectionError, ConanException, NotFoundException, \
    NoRestV2Available, PackageNotFoundException
from conans.paths import EXPORT_SOURCES_DIR_OLD, \
    EXPORT_SOURCES_TGZ_NAME, EXPORT_TGZ_NAME, PACKAGE_TGZ_NAME, rm_conandir
from conans.search.search import filter_packages
from conans.util import progress_bar
from conans.util.env_reader import get_env
from conans.util.files import make_read_only, mkdir, rmdir, tar_extract, touch_folder
from conans.util.log import logger
# FIXME: Eventually, when all output is done, tracer functions should be moved to the recorder class
from conans.util.tracer import (log_package_download,
                                log_recipe_download, log_recipe_sources_download,
                                log_uncompressed_file)


class RemoteManager(object):
    """ Will handle the remotes to get recipes, packages etc """

    def __init__(self, cache, auth_manager, output, hook_manager):
        self._cache = cache
        self._output = output
        self._auth_manager = auth_manager
        self._hook_manager = hook_manager

    def check_credentials(self, remote):
        self._call_remote(remote, "check_credentials")

    def get_recipe_snapshot(self, ref, remote):
        assert ref.revision, "get_recipe_snapshot requires revision"
        return self._call_remote(remote, "get_recipe_snapshot", ref)

    def get_package_snapshot(self, pref, remote):
        assert pref.ref.revision, "upload_package requires RREV"
        assert pref.revision, "get_package_snapshot requires PREV"
        return self._call_remote(remote, "get_package_snapshot", pref)

    def upload_recipe(self, ref, files_to_upload, deleted, remote, retry, retry_wait):
        assert ref.revision, "upload_recipe requires RREV"
        self._call_remote(remote, "upload_recipe", ref, files_to_upload, deleted,
                          retry, retry_wait)

    def upload_package(self, pref, files_to_upload, deleted, remote, retry, retry_wait):
        assert pref.ref.revision, "upload_package requires RREV"
        assert pref.revision, "upload_package requires PREV"
        self._call_remote(remote, "upload_package", pref,
                          files_to_upload, deleted, retry, retry_wait)

    def get_recipe_manifest(self, ref, remote):
        ref = self._resolve_latest_ref(ref, remote)
        return self._call_remote(remote, "get_recipe_manifest", ref), ref

    def get_package_manifest(self, pref, remote):
        pref = self._resolve_latest_pref(pref, remote)
        return self._call_remote(remote, "get_package_manifest", pref), pref

    def get_package_info(self, pref, remote):
        """ Read a package ConanInfo from remote
        """
        pref = self._resolve_latest_pref(pref, remote)
        return self._call_remote(remote, "get_package_info", pref), pref

    def get_recipe(self, ref, remote):
        """
        Read the conans from remotes
        Will iterate the remotes to find the conans unless remote was specified

        returns (dict relative_filepath:abs_path , remote_name)"""

        self._hook_manager.execute("pre_download_recipe", reference=ref, remote=remote)
        dest_folder = self._cache.package_layout(ref).export()
        rmdir(dest_folder)

        ref = self._resolve_latest_ref(ref, remote)

        t1 = time.time()
        zipped_files = self._call_remote(remote, "get_recipe", ref, dest_folder)
        duration = time.time() - t1
        log_recipe_download(ref, duration, remote.name, zipped_files)

        unzip_and_get_files(zipped_files, dest_folder, EXPORT_TGZ_NAME, output=self._output)
        # Make sure that the source dir is deleted
        package_layout = self._cache.package_layout(ref)
        rm_conandir(package_layout.source())
        touch_folder(dest_folder)
        conanfile_path = package_layout.conanfile()

        with package_layout.update_metadata() as metadata:
            metadata.recipe.revision = ref.revision

        self._hook_manager.execute("post_download_recipe", conanfile_path=conanfile_path,
                                   reference=ref, remote=remote)

        return ref

    def get_recipe_sources(self, ref, export_folder, export_sources_folder, remote):
        assert ref.revision, "get_recipe_sources requires RREV"
        t1 = time.time()

        zipped_files = self._call_remote(remote, "get_recipe_sources", ref, export_folder)
        if not zipped_files:
            mkdir(export_sources_folder)  # create the folder even if no source files
            return

        duration = time.time() - t1
        log_recipe_sources_download(ref, duration, remote.name, zipped_files)

        unzip_and_get_files(zipped_files, export_sources_folder, EXPORT_SOURCES_TGZ_NAME,
                            output=self._output)
        # REMOVE in Conan 2.0
        c_src_path = os.path.join(export_sources_folder, EXPORT_SOURCES_DIR_OLD)
        if os.path.exists(c_src_path):
            merge_directories(c_src_path, export_sources_folder)
            rmdir(c_src_path)
        touch_folder(export_sources_folder)

    def get_package(self, pref, dest_folder, remote, output, recorder):

        conanfile_path = self._cache.package_layout(pref.ref).conanfile()
        self._hook_manager.execute("pre_download_package", conanfile_path=conanfile_path,
                                   reference=pref.ref, package_id=pref.id, remote=remote)
        output.info("Retrieving package %s from remote '%s' " % (pref.id, remote.name))
        rm_conandir(dest_folder)  # Remove first the destination folder
        t1 = time.time()
        try:
            pref = self._resolve_latest_pref(pref, remote)
            snapshot = self._call_remote(remote, "get_package_snapshot", pref)
            if not is_package_snapshot_complete(snapshot):
                raise PackageNotFoundException(pref)
            zipped_files = self._call_remote(remote, "get_package", pref, dest_folder)

            with self._cache.package_layout(pref.ref).update_metadata() as metadata:
                metadata.packages[pref.id].revision = pref.revision
                metadata.packages[pref.id].recipe_revision = pref.ref.revision

            duration = time.time() - t1
            log_package_download(pref, duration, remote, zipped_files)
            unzip_and_get_files(zipped_files, dest_folder, PACKAGE_TGZ_NAME, output=self._output)
            # Issue #214 https://github.com/conan-io/conan/issues/214
            touch_folder(dest_folder)
            if get_env("CONAN_READ_ONLY_CACHE", False):
                make_read_only(dest_folder)
            recorder.package_downloaded(pref, remote.url)
            output.success('Package installed %s' % pref.id)
        except NotFoundException:
            raise PackageNotFoundException(pref)
        except BaseException as e:
            output.error("Exception while getting package: %s" % str(pref.id))
            output.error("Exception: %s %s" % (type(e), str(e)))
            try:
                output.warn("Trying to remove package folder: %s" % dest_folder)
                rmdir(dest_folder)
            except OSError as e:
                raise ConanException("%s\n\nCouldn't remove folder '%s', might be busy or open. "
                                     "Close any app using it, and retry" % (str(e), dest_folder))
            raise
        self._hook_manager.execute("post_download_package", conanfile_path=conanfile_path,
                                   reference=pref.ref, package_id=pref.id, remote=remote)

        return pref

    def search_recipes(self, remote, pattern=None, ignorecase=True):
        """
        Search exported conans information from remotes

        returns (dict str(ref): {packages_info}"""
        return self._call_remote(remote, "search", pattern, ignorecase)

    def search_packages(self, remote, ref, query):
        packages = self._call_remote(remote, "search_packages", ref, query)
        packages = filter_packages(query, packages)
        return packages

    def remove(self, ref, remote):
        """
        Removed conans or packages from remote
        """
        return self._call_remote(remote, "remove", ref)

    def remove_packages(self, ref, remove_ids, remote):
        """
        Removed conans or packages from remote
        """
        return self._call_remote(remote, "remove_packages", ref, remove_ids)

    def get_recipe_path(self, ref, path, remote):
        return self._call_remote(remote, "get_recipe_path", ref, path)

    def get_package_path(self, pref, path, remote):
        return self._call_remote(remote, "get_package_path", pref, path)

    def authenticate(self, remote, name, password):
        return self._call_remote(remote, 'authenticate', name, password)

    def get_recipe_revisions(self, ref, remote):
        return self._call_remote(remote, "get_recipe_revisions", ref)

    def get_package_revisions(self, pref, remote):
        revisions = self._call_remote(remote, "get_package_revisions", pref)
        return revisions

    def get_latest_recipe_revision(self, ref, remote):
        revision = self._call_remote(remote, "get_latest_recipe_revision", ref)
        return revision

    def get_latest_package_revision(self, pref, remote):
        revision = self._call_remote(remote, "get_latest_package_revision", pref)
        return revision

    def _resolve_latest_ref(self, ref, remote):
        if ref.revision is None:
            try:
                ref = self.get_latest_recipe_revision(ref, remote)
            except NoRestV2Available:
                ref = ref.copy_with_rev(DEFAULT_REVISION_V1)
        return ref

    def _resolve_latest_pref(self, pref, remote):
        if pref.revision is None:
            try:
                pref = self.get_latest_package_revision(pref, remote)
            except NoRestV2Available:
                pref = pref.copy_with_revs(pref.ref.revision, DEFAULT_REVISION_V1)
        return pref

    def _call_remote(self, remote, method, *argc, **argv):
        assert(isinstance(remote, Remote))
        self._auth_manager.remote = remote
        try:
            return getattr(self._auth_manager, method)(*argc, **argv)
        except ConnectionError as exc:
            raise ConanConnectionError("%s\n\nUnable to connect to %s=%s"
                                       % (str(exc), remote.name, remote.url))
        except ConanException as exc:
            exc.remote = remote
            raise
        except Exception as exc:
            logger.error(traceback.format_exc())
            raise ConanException(exc, remote=remote)


def is_package_snapshot_complete(snapshot):
    integrity = True
    for keyword in ["conaninfo", "conanmanifest", "conan_package"]:
        if not any(keyword in key for key in snapshot):
            integrity = False
            break
    return integrity


def check_compressed_files(tgz_name, files):
    bare_name = os.path.splitext(tgz_name)[0]
    for f in files:
        if f == tgz_name:
            continue
        if bare_name == os.path.splitext(f)[0]:
            raise ConanException("This Conan version is not prepared to handle '%s' file format. "
                                 "Please upgrade conan client." % f)


def unzip_and_get_files(files, destination_dir, tgz_name, output):
    """Moves all files from package_files, {relative_name: tmp_abs_path}
    to destination_dir, unzipping the "tgz_name" if found"""

    tgz_file = files.pop(tgz_name, None)
    check_compressed_files(tgz_name, files)
    if tgz_file:
        uncompress_file(tgz_file, destination_dir, output=output)
        os.remove(tgz_file)


def uncompress_file(src_path, dest_folder, output):
    t1 = time.time()
    try:
        with progress_bar.open_binary(src_path, desc="Decompressing %s" % os.path.basename(src_path),
                                      output=output) as file_handler:
            tar_extract(file_handler, dest_folder)
    except Exception as e:
        error_msg = "Error while downloading/extracting files to %s\n%s\n" % (dest_folder, str(e))
        # try to remove the files
        try:
            if os.path.exists(dest_folder):
                shutil.rmtree(dest_folder)
                error_msg += "Folder removed"
        except Exception:
            error_msg += "Folder not removed, files/package might be damaged, remove manually"
        raise ConanException(error_msg)

    duration = time.time() - t1
    log_uncompressed_file(src_path, duration, dest_folder)



import unittest

from conans.client.generators.boostbuild import BoostBuildGenerator
from conans.model.build_info import CppInfo
from conans.model.conan_file import ConanFile
from conans.model.env_info import EnvValues
from conans.model.ref import ConanFileReference
from conans.model.settings import Settings
from conans.test.utils.tools import TestBufferConanOutput


class BoostJamGeneratorTest(unittest.TestCase):

    def variables_setup_test(self):

        conanfile = ConanFile(TestBufferConanOutput(), None)
        conanfile.initialize(Settings({}), EnvValues())

        ref = ConanFileReference.loads("MyPkg/0.1@lasote/stables")
        cpp_info = CppInfo("dummy_root_folder1")
        cpp_info.defines = ["MYDEFINE1"]
        cpp_info.cflags.append("-Flag1=23")
        cpp_info.version = "1.3"
        cpp_info.description = "My cool description"
        cpp_info.libs = ["MyLib1"]

        conanfile.deps_cpp_info.update(cpp_info, ref.name)
        ref = ConanFileReference.loads("MyPkg2/0.1@lasote/stables")
        cpp_info = CppInfo("dummy_root_folder2")
        cpp_info.libs = ["MyLib2"]
        cpp_info.defines = ["MYDEFINE2"]
        cpp_info.version = "2.3"
        cpp_info.exelinkflags = ["-exelinkflag"]
        cpp_info.sharedlinkflags = ["-sharedlinkflag"]
        cpp_info.cxxflags = ["-cxxflag"]
        cpp_info.public_deps = ["MyPkg"]
        cpp_info.lib_paths.extend(["Path\\with\\slashes", "regular/path/to/dir"])
        cpp_info.include_paths.extend(["other\\Path\\with\\slashes", "other/regular/path/to/dir"])
        conanfile.deps_cpp_info.update(cpp_info, ref.name)
        generator = BoostBuildGenerator(conanfile)

        self.assertEqual(generator.content, """lib MyLib1 :
	: # requirements
	<name>MyLib1
	: # default-build
	: # usage-requirements
	<define>MYDEFINE1
	<cflags>-Flag1=23
	;

lib MyLib2 :
	: # requirements
	<name>MyLib2
	<search>Path/with/slashes
	<search>regular/path/to/dir
	: # default-build
	: # usage-requirements
	<define>MYDEFINE2
	<include>other/Path/with/slashes
	<include>other/regular/path/to/dir
	<cxxflags>-cxxflag
	<ldflags>-sharedlinkflag
	;

alias conan-deps :
	MyLib1
	MyLib2
;
""")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest

from parameterized import parameterized

from conans.client.conf import default_settings_yml
from conans.model.info import ConanInfo
from conans.model.settings import Settings


class VSToolsetCompatibleTest(unittest.TestCase):
    @parameterized.expand([("15", "v142", "16"),
                           ("14", "v141", "15"),
                           ("15", "v140", "14"),
                           ("11", "v120", "12"),
                           ("12", "v110", "11"),
                           ("9", "v100", "10"),
                           ("10", "v90", "9")])
    def test_compatible(self, initial_version, toolset, expected_version):
        info = ConanInfo()
        settings = Settings.loads(default_settings_yml)
        info.settings = settings
        settings.compiler = "Visual Studio"
        settings.compiler.toolset = toolset
        settings.compiler.version = initial_version
        info.full_settings = info.settings
        info.vs_toolset_compatible()
        self.assertEqual(info.settings.compiler.version, expected_version)
        self.assertIsNone(info.settings.get_safe("compiler.toolset"))

    @parameterized.expand([("16", "v141_xp"),
                           ("14", "v141_xp"),
                           ("15", "v140_xp"),
                           ("11", "v120_xp"),
                           ("12", "v110_xp"),
                           ("9", "LLVM-vs2012_xp"),
                           ("10", "LLVM-vs2013_xp")])
    def test_incompatible(self, initial_version, toolset):
        info = ConanInfo()
        settings = Settings.loads(default_settings_yml)
        info.settings = settings
        settings.compiler = "Visual Studio"
        settings.compiler.toolset = toolset
        settings.compiler.version = initial_version
        info.full_settings = info.settings
        info.vs_toolset_compatible()
        self.assertEqual(info.settings.compiler.version, initial_version)
        self.assertEqual(info.settings.compiler.toolset, toolset)



import os
from collections import namedtuple

from conans import Options
from conans.model.conan_file import ConanFile
from conans.model.options import PackageOptions
from conans.test.utils.tools import TestBufferConanOutput


class MockSettings(object):

    def __init__(self, values):
        self.values = values

    def get_safe(self, value):
        return self.values.get(value, None)


MockOptions = MockSettings


class MockDepsCppInfo(object):

    def __init__(self):
        self.include_paths = []
        self.lib_paths = []
        self.libs = []
        self.defines = []
        self.cflags = []
        self.cxxflags = []
        self.sharedlinkflags = []
        self.exelinkflags = []
        self.sysroot = ""


class MockConanfile(ConanFile):

    def __init__(self, settings, options=None, runner=None):
        self.deps_cpp_info = MockDepsCppInfo()
        self.settings = settings
        self.runner = runner
        self.options = options or MockOptions({})
        self.generators = []
        self.output = TestBufferConanOutput()

        self.should_configure = True
        self.should_build = True
        self.should_install = True
        self.should_test = True

        self.package_folder = None

    def run(self, *args, **kwargs):
        if self.runner:
            kwargs["output"] = None
            self.runner(*args, **kwargs)


class TestConanFile(object):
    def __init__(self, name="Hello", version="0.1", settings=None, requires=None, options=None,
                 default_options=None, package_id=None, build_requires=None, info=None,
                 private_requires=None, private_first=False):
        self.name = name
        self.version = version
        self.settings = settings
        self.requires = requires
        self.private_requires = private_requires
        self.build_requires = build_requires
        self.options = options
        self.default_options = default_options
        self.package_id = package_id
        self.info = info
        self.private_first = private_first

    def __repr__(self):
        base = """from conans import ConanFile

class {name}Conan(ConanFile):
    name = "{name}"
    version = "{version}"
""".format(name=self.name, version=self.version)
        if self.settings:
            base += "    settings = %s\n" % self.settings
        if self.requires or self.private_requires:
            if self.private_first:
                reqs_list = ["('%s', 'private')" % r for r in self.private_requires or []]
                reqs_list.extend(['"%s"' % r for r in self.requires or []])
            else:
                reqs_list = ['"%s"' % r for r in self.requires or []]
                reqs_list.extend(["('%s', 'private')" % r for r in self.private_requires or []])
            reqs_list.append("")
            base += "    requires = %s\n" % (", ".join(reqs_list))
        if self.build_requires:
            base += "    build_requires = %s\n" % (", ".join('"%s"' % r
                                                             for r in self.build_requires))
        if self.options:
            base += "    options = %s\n" % str(self.options)
        if self.default_options:
            if isinstance(self.default_options, str):
                base += "    default_options = '%s'\n" % str(self.default_options)
            else:
                base += "    default_options = %s\n" % str(self.default_options)
        if self.package_id:
            base += "    def package_id(self):\n        %s\n" % self.package_id
        if self.info:
            base += """
    def package_info(self):
        self.cpp_info.libs = ["mylib{name}{version}lib"]
        self.env_info.MYENV = ["myenv{name}{version}env"]
""".format(name=self.name, version=self.version)
        return base


class ConanFileMock(ConanFile):

    def __init__(self, shared=None, options=None, options_values=None):
        options = options or ""
        self.command = None
        self.path = None
        self.source_folder = self.build_folder = "."
        self.settings = None
        self.options = Options(PackageOptions.loads(options))
        if options_values:
            for var, value in options_values.items():
                self.options._data[var] = value
        self.deps_cpp_info = namedtuple("deps_cpp_info", "sysroot")("/path/to/sysroot")
        self.output = TestBufferConanOutput()
        self.in_local_cache = False
        self.install_folder = "myinstallfolder"
        if shared is not None:
            self.options = namedtuple("options", "shared")(shared)
        self.should_configure = True
        self.should_build = True
        self.should_install = True
        self.should_test = True
        self.generators = []
        self.captured_env = {}

    def run(self, command):
        self.command = command
        self.path = os.environ["PATH"]
        self.captured_env = {key: value for key, value in os.environ.items()}



import json
import os

from conans.errors import ConanException
from conans.model.options import OptionsValues
from conans.model.ref import ConanFileReference
from conans.tools import save
from conans.util.files import load
from conans.model.graph_lock import GraphLockFile, LOCKFILE


GRAPH_INFO_FILE = "graph_info.json"


class GraphInfo(object):

    def __init__(self, profile=None, options=None, root_ref=None):
        # This field is a temporary hack, to store dependencies options for the local flow
        self.options = options
        self.root = root_ref
        self.profile = profile
        self.graph_lock = None

    @staticmethod
    def load(path):
        if not path:
            raise IOError("Invalid path")
        p = os.path.join(path, GRAPH_INFO_FILE)
        content = load(p)
        try:
            graph_info = GraphInfo.loads(content)
            return graph_info
        except Exception as e:
            raise ConanException("Error parsing GraphInfo from file '{}': {}".format(p, e))

    @staticmethod
    def loads(text):
        graph_json = json.loads(text)
        try:
            options = graph_json["options"]
        except KeyError:
            options = None
        else:
            options = OptionsValues(options)
        root = graph_json.get("root", {"name": None, "version": None, "user": None, "channel": None})
        root_ref = ConanFileReference(root["name"], root["version"], root["user"], root["channel"],
                                      validate=False)

        return GraphInfo(options=options, root_ref=root_ref)

    def save(self, folder, filename=None):
        filename = filename or GRAPH_INFO_FILE
        p = os.path.join(folder, filename)
        serialized_graph_str = self._dumps()
        save(p, serialized_graph_str)

        # A bit hacky, but to avoid repetition by now
        graph_lock_file = GraphLockFile(self.profile, self.graph_lock)
        graph_lock_file.save(os.path.join(folder, LOCKFILE))

    def save_lock(self, lockfile):
        graph_lock_file = GraphLockFile(self.profile, self.graph_lock)
        graph_lock_file.save(lockfile)

    def _dumps(self):
        result = {}
        if self.options is not None:
            result["options"] = self.options.as_list()
        result["root"] = {"name": self.root.name,
                          "version": self.root.version,
                          "user": self.root.user,
                          "channel": self.root.channel}
        return json.dumps(result, indent=True)



# coding=utf-8
import os
from collections import OrderedDict

import six
from six.moves import configparser

from conans.errors import ConanException
from conans.model.ref import ConanFileReference, check_valid_ref
from conans.util.files import load
from conans.util.templates import render_layout_file

DEFAULT_LAYOUT_FILE = "default"
LAYOUTS_FOLDER = 'layouts'


def get_editable_abs_path(path, cwd, cache_folder):
    # Check the layout file exists, is correct, and get its abs-path
    if path:
        layout_abs_path = os.path.normpath(os.path.join(cwd, path))
        if not os.path.isfile(layout_abs_path):
            layout_abs_path = os.path.join(cache_folder, LAYOUTS_FOLDER, path)
        if not os.path.isfile(layout_abs_path):
            raise ConanException("Couldn't find layout file: %s" % path)
        return layout_abs_path

    # Default only in cache
    layout_default_path = os.path.join(cache_folder, LAYOUTS_FOLDER, DEFAULT_LAYOUT_FILE)
    if os.path.isfile(layout_default_path):
        return layout_default_path


class EditableLayout(object):
    BUILD_FOLDER = "build_folder"
    SOURCE_FOLDER = "source_folder"
    cpp_info_dirs = ['includedirs', 'libdirs', 'resdirs', 'bindirs', 'builddirs', 'srcdirs']
    folders = [BUILD_FOLDER, SOURCE_FOLDER]

    def __init__(self, filepath):
        self._filepath = filepath

    def folder(self, ref, name, settings, options):
        _, folders = self._load_data(ref, settings=settings, options=options)
        try:
            path = folders.get(str(ref)) or folders.get(None) or {}
            return path[name]
        except KeyError:
            return None

    @staticmethod
    def _work_on_item(value):
        value = value.replace('\\', '/')
        return value

    def _parse_layout_file(self, ref, settings, options):
        content = load(self._filepath)
        try:
            content = render_layout_file(content, ref=ref, settings=settings, options=options)

            parser = configparser.ConfigParser(allow_no_value=True)
            parser.optionxform = str
            if six.PY3:
                parser.read_string(content)
            else:
                parser.readfp(six.StringIO(content))
        except (configparser.Error, ConanException) as e:
            raise ConanException("Error parsing layout file '%s' (for reference '%s')\n%s" %
                                 (self._filepath, str(ref), str(e)))

        return parser

    def _load_data(self, ref, settings, options):
        parser = self._parse_layout_file(ref, settings, options)

        # Build a convenient data structure
        data = OrderedDict()
        folders = {}
        for section in parser.sections():
            reference, section_name = section.rsplit(":", 1) if ':' in section else (None, section)

            if section_name in EditableLayout.folders:
                items = [k for k, _ in parser.items(section)] or [""]
                if len(items) > 1:
                    raise ConanException("'%s' with more than one value in layout file: %s"
                                         % (section_name, self._filepath))
                folders.setdefault(reference, {})[section_name] = self._work_on_item(items[0])
                continue

            if section_name not in EditableLayout.cpp_info_dirs:
                raise ConanException("Wrong cpp_info field '%s' in layout file: %s"
                                     % (section_name, self._filepath))
            if reference:
                if not check_valid_ref(reference):
                    raise ConanException("Wrong package reference '%s' in layout file: %s"
                                         % (reference, self._filepath))
                else:
                    r = ConanFileReference.loads(reference, validate=True)
                    if r.revision:
                        raise ConanException("Don't provide revision in Editable layouts")

            data.setdefault(reference, {})[section_name] =\
                [self._work_on_item(k) for k, _ in parser.items(section)]
        return data, folders

    def apply_to(self, ref, cpp_info, settings=None, options=None):
        data, _ = self._load_data(ref, settings=settings, options=options)

        # Apply the data to the cpp_info
        data = data.get(str(ref)) or data.get(None) or {}

        try:
            for key, items in data.items():
                setattr(cpp_info, key, items)
        except Exception as e:
            raise ConanException("Error applying layout in '%s': %s" % (str(ref), str(e)))



import os
import sys
import textwrap
import unittest
from collections import OrderedDict

import six
from mock import Mock
from mock.mock import call
from parameterized import parameterized

from conans.client.graph.python_requires import ConanPythonRequire
from conans.client.loader import ConanFileLoader, ConanFileTextLoader,\
    _parse_conanfile
from conans.client.tools.files import chdir
from conans.errors import ConanException
from conans.model.options import OptionsValues
from conans.model.profile import Profile
from conans.model.requires import Requirements
from conans.model.settings import Settings
from conans.test.utils.test_files import temp_folder
from conans.test.utils.tools import test_processed_profile,\
    TestBufferConanOutput
from conans.util.files import save


class ConanLoaderTest(unittest.TestCase):

    def inherit_short_paths_test(self):
        loader = ConanFileLoader(None, TestBufferConanOutput(), ConanPythonRequire(None, None))
        tmp_dir = temp_folder()
        conanfile_path = os.path.join(tmp_dir, "conanfile.py")
        conanfile = """from base_recipe import BasePackage
class Pkg(BasePackage):
    pass
"""
        base_recipe = """from conans import ConanFile
class BasePackage(ConanFile):
    short_paths = True
"""
        save(conanfile_path, conanfile)
        save(os.path.join(tmp_dir, "base_recipe.py"), base_recipe)
        conan_file = loader.load_class(conanfile_path)
        self.assertEqual(conan_file.short_paths, True)

        result = loader.load_consumer(conanfile_path,
                                      processed_profile=test_processed_profile())
        self.assertEqual(result.short_paths, True)

    def requires_init_test(self):
        loader = ConanFileLoader(None, TestBufferConanOutput(), ConanPythonRequire(None, None))
        tmp_dir = temp_folder()
        conanfile_path = os.path.join(tmp_dir, "conanfile.py")
        conanfile = """from conans import ConanFile
class MyTest(ConanFile):
    requires = {}
    def requirements(self):
        self.requires("MyPkg/0.1@user/channel")
"""
        for requires in ("''", "[]", "()", "None"):
            save(conanfile_path, conanfile.format(requires))
            result = loader.load_consumer(conanfile_path,
                                          processed_profile=test_processed_profile())
            result.requirements()
            self.assertEqual("MyPkg/0.1@user/channel", str(result.requires))

    def conanfile_txt_errors_test(self):
        # Valid content
        file_content = '''[requires}
OpenCV/2.4.10@phil/stable # My requirement for CV
'''
        with six.assertRaisesRegex(self, ConanException, "Bad syntax"):
            ConanFileTextLoader(file_content)

        file_content = '{hello}'
        with six.assertRaisesRegex(self, ConanException, "Unexpected line"):
            ConanFileTextLoader(file_content)

        file_content = '[imports]\nhello'
        with six.assertRaisesRegex(self, ConanException, "Invalid imports line: hello"):
            ConanFileTextLoader(file_content).imports_method(None)

        file_content = '[imports]\nbin, * -> bin @ kk=3 '
        with six.assertRaisesRegex(self, ConanException, "Unknown argument kk"):
            ConanFileTextLoader(file_content).imports_method(None)

    def plain_text_parser_test(self):
        # Valid content
        file_content = '''[requires]
OpenCV/2.4.10@phil/stable # My requirement for CV
OpenCV2/2.4.10@phil/stable #
OpenCV3/2.4.10@phil/stable
[generators]
one # My generator for this
two
[options]
OpenCV:use_python=True # Some option
OpenCV:other_option=False
OpenCV2:use_python2=1
OpenCV2:other_option=Cosa #
'''
        parser = ConanFileTextLoader(file_content)
        exp = ['OpenCV/2.4.10@phil/stable',
               'OpenCV2/2.4.10@phil/stable',
               'OpenCV3/2.4.10@phil/stable']
        self.assertEqual(parser.requirements, exp)

    def revision_parsing_test(self):
        # Valid content
        file_content = '''[requires]
OpenCV/2.4.10@user/stable#RREV1 # My requirement for CV
'''
        parser = ConanFileTextLoader(file_content)
        exp = ['OpenCV/2.4.10@user/stable#RREV1']
        self.assertEqual(parser.requirements, exp)

    def load_conan_txt_test(self):
        file_content = '''[requires]
OpenCV/2.4.10@phil/stable
OpenCV2/2.4.10@phil/stable
[build_requires]
MyPkg/1.0.0@phil/stable
[generators]
one
two
[imports]
OpenCV/bin, * -> ./bin # I need this binaries
OpenCV/lib, * -> ./lib
[options]
OpenCV:use_python=True
OpenCV:other_option=False
OpenCV2:use_python2=1
OpenCV2:other_option=Cosa
'''
        tmp_dir = temp_folder()
        file_path = os.path.join(tmp_dir, "file.txt")
        save(file_path, file_content)
        loader = ConanFileLoader(None, TestBufferConanOutput(), None)
        ret = loader.load_conanfile_txt(file_path, test_processed_profile())
        options1 = OptionsValues.loads("""OpenCV:use_python=True
OpenCV:other_option=False
OpenCV2:use_python2=1
OpenCV2:other_option=Cosa""")
        requirements = Requirements()
        requirements.add("OpenCV/2.4.10@phil/stable")
        requirements.add("OpenCV2/2.4.10@phil/stable")
        build_requirements = []
        build_requirements.append("MyPkg/1.0.0@phil/stable")

        self.assertEqual(ret.requires, requirements)
        self.assertEqual(ret.build_requires, build_requirements)
        self.assertEqual(ret.generators, ["one", "two"])
        self.assertEqual(ret.options.values.dumps(), options1.dumps())

        ret.copy = Mock()
        ret.imports()

        self.assertTrue(ret.copy.call_args_list, [('*', './bin', 'OpenCV/bin'),
                                                  ('*', './lib', 'OpenCV/lib')])

        # Now something that fails
        file_content = '''[requires]
OpenCV/2.4.104phil/stable
'''
        tmp_dir = temp_folder()
        file_path = os.path.join(tmp_dir, "file.txt")
        save(file_path, file_content)
        loader = ConanFileLoader(None, TestBufferConanOutput(), None)
        with six.assertRaisesRegex(self, ConanException, "The reference has too many '/'"):
            loader.load_conanfile_txt(file_path, test_processed_profile())

        file_content = '''[requires]
OpenCV/2.4.10@phil/stable111111111111111111111111111111111111111111111111111111111111111
[imports]
OpenCV/bin/* - ./bin
'''
        tmp_dir = temp_folder()
        file_path = os.path.join(tmp_dir, "file.txt")
        save(file_path, file_content)
        loader = ConanFileLoader(None, TestBufferConanOutput(), None)
        with six.assertRaisesRegex(self, ConanException, "is too long. Valid names must contain"):
            loader.load_conanfile_txt(file_path, test_processed_profile())

    def load_imports_arguments_test(self):
        file_content = '''
[imports]
OpenCV/bin, * -> ./bin # I need this binaries
OpenCV/lib, * -> ./lib @ root_package=Pkg
OpenCV/data, * -> ./data @ root_package=Pkg, folder=True # Irrelevant
docs, * -> ./docs @ root_package=Pkg, folder=True, ignore_case=True, excludes="a b c" # Other
licenses, * -> ./licenses @ root_package=Pkg, folder=True, ignore_case=True, excludes="a b c", keep_path=False # Other
'''
        tmp_dir = temp_folder()
        file_path = os.path.join(tmp_dir, "file.txt")
        save(file_path, file_content)
        loader = ConanFileLoader(None, TestBufferConanOutput(), None)
        ret = loader.load_conanfile_txt(file_path, test_processed_profile())

        ret.copy = Mock()
        ret.imports()
        expected = [call(u'*', u'./bin', u'OpenCV/bin', None, False, False, None, True),
                    call(u'*', u'./lib', u'OpenCV/lib', u'Pkg', False, False, None, True),
                    call(u'*', u'./data', u'OpenCV/data', u'Pkg', True, False, None, True),
                    call(u'*', u'./docs', u'docs', u'Pkg', True, True, [u'"a', u'b', u'c"'], True),
                    call(u'*', u'./licenses', u'licenses', u'Pkg', True, True, [u'"a', u'b', u'c"'],
                         False)]
        self.assertEqual(ret.copy.call_args_list, expected)

    def test_package_settings(self):
        # CREATE A CONANFILE TO LOAD
        tmp_dir = temp_folder()
        conanfile_path = os.path.join(tmp_dir, "conanfile.py")
        conanfile = """from conans import ConanFile
class MyTest(ConanFile):
    requires = {}
    name = "MyPackage"
    version = "1.0"
    settings = "os"
"""
        save(conanfile_path, conanfile)

        # Apply windows for MyPackage
        profile = Profile()
        profile.processed_settings = Settings({"os": ["Windows", "Linux"]})
        profile.package_settings = {"MyPackage": OrderedDict([("os", "Windows")])}
        loader = ConanFileLoader(None, TestBufferConanOutput(), ConanPythonRequire(None, None))

        recipe = loader.load_consumer(conanfile_path, profile)
        self.assertEqual(recipe.settings.os, "Windows")

        # Apply Linux for MyPackage
        profile = Profile()
        profile.processed_settings = Settings({"os": ["Windows", "Linux"]})
        profile.package_settings = {"MyPackage": OrderedDict([("os", "Linux")])}
        recipe = loader.load_consumer(conanfile_path, profile)
        self.assertEqual(recipe.settings.os, "Linux")

        # If the package name is different from the conanfile one, it wont apply
        profile = Profile()
        profile.processed_settings = Settings({"os": ["Windows", "Linux"]})
        profile.package_settings = {"OtherPACKAGE": OrderedDict([("os", "Linux")])}
        recipe = loader.load_consumer(conanfile_path, profile)
        self.assertIsNone(recipe.settings.os.value)


class ImportModuleLoaderTest(unittest.TestCase):
    @staticmethod
    def _create_and_load(myfunc, value, subdir_name, add_subdir_init):
        subdir_content = textwrap.dedent("""
            def get_value():
                return {value}
            def {myfunc}():
                return "{myfunc}"
        """)

        side_content = textwrap.dedent("""
            def get_side_value():
                return {value}
            def side_{myfunc}():
                return "{myfunc}"
        """)

        conanfile = textwrap.dedent("""
            import pickle
            from {subdir}.api import get_value, {myfunc}
            from file import get_side_value, side_{myfunc}
            from fractions import Fraction
            def conanfile_func():
                return get_value(), {myfunc}(), get_side_value(), side_{myfunc}(), str(Fraction(1,1))
        """)
        expected_return = (value, myfunc, value, myfunc, "1")

        tmp = temp_folder()
        with chdir(tmp):
            save("conanfile.py", conanfile.format(value=value, myfunc=myfunc, subdir=subdir_name))
            save("file.py", side_content.format(value=value, myfunc=myfunc))
            save("{}/api.py".format(subdir_name), subdir_content.format(value=value, myfunc=myfunc))
            if add_subdir_init:
                save("__init__.py", "")
                save("{}/__init__.py".format(subdir_name), "")

        loaded, module_id = _parse_conanfile(os.path.join(tmp, "conanfile.py"))
        return loaded, module_id, expected_return

    @parameterized.expand([(True, False), (False, True), (False, False)])
    @unittest.skipIf(six.PY2, "Python 2 requires __init__.py file in modules")
    def test_py3_recipe_colliding_init_filenames(self, sub1, sub2):
        myfunc1, value1 = "recipe1", 42
        myfunc2, value2 = "recipe2", 23
        loaded1, module_id1, exp_ret1 = self._create_and_load(myfunc1, value1, "subdir", sub1)
        loaded2, module_id2, exp_ret2 = self._create_and_load(myfunc2, value2, "subdir", sub2)

        self.assertNotEqual(module_id1, module_id2)
        self.assertEqual(loaded1.conanfile_func(), exp_ret1)
        self.assertEqual(loaded2.conanfile_func(), exp_ret2)

    def test_recipe_colliding_filenames(self):
        myfunc1, value1 = "recipe1", 42
        myfunc2, value2 = "recipe2", 23
        loaded1, module_id1, exp_ret1 = self._create_and_load(myfunc1, value1, "subdir", True)
        loaded2, module_id2, exp_ret2 = self._create_and_load(myfunc2, value2, "subdir", True)

        self.assertNotEqual(module_id1, module_id2)
        self.assertEqual(loaded1.conanfile_func(), exp_ret1)
        self.assertEqual(loaded2.conanfile_func(), exp_ret2)

    @parameterized.expand([(True, ), (False, )])
    def test_wrong_imports(self, add_subdir_init):
        myfunc1, value1 = "recipe1", 42

        # Item imported does not exist, but file exists
        with six.assertRaisesRegex(self, ConanException, "Unable to load conanfile in"):
            self._create_and_load(myfunc1, value1, "requests", add_subdir_init)

        # File does not exists in already existing module
        with six.assertRaisesRegex(self, ConanException, "Unable to load conanfile in"):
            self._create_and_load(myfunc1, value1, "conans", add_subdir_init)

    def test_helpers_python_library(self):
        mylogger = """
value = ""
def append(data):
    global value
    value += data
"""
        temp = temp_folder()
        save(os.path.join(temp, "myconanlogger.py"), mylogger)

        conanfile = "import myconanlogger"
        temp1 = temp_folder()
        save(os.path.join(temp1, "conanfile.py"), conanfile)
        temp2 = temp_folder()
        save(os.path.join(temp2, "conanfile.py"), conanfile)

        try:
            sys.path.append(temp)
            loaded1, _ = _parse_conanfile(os.path.join(temp1, "conanfile.py"))
            loaded2, _ = _parse_conanfile(os.path.join(temp2, "conanfile.py"))
            self.assertIs(loaded1.myconanlogger, loaded2.myconanlogger)
            self.assertIs(loaded1.myconanlogger.value, loaded2.myconanlogger.value)
        finally:
            sys.path.remove(temp)



from math import floor

import time
import unittest

from conans.server.revision_list import RevisionList
from conans.util.dates import from_timestamp_to_iso8601


class RevisionListTest(unittest.TestCase):

    def test_remove_latest(self):
        rev = RevisionList()
        rev.add_revision("rev1")
        rev.add_revision("rev2")

        dumped = rev.dumps()
        loaded = RevisionList.loads(dumped)
        self.assertEqual(rev, loaded)
        self.assertEqual(loaded.latest_revision().revision, "rev2")

        loaded.remove_revision("rev2")
        self.assertEqual(loaded.latest_revision().revision, "rev1")

    def test_remove_non_latest(self):
        rev = RevisionList()
        rev.add_revision("rev1")
        rev.add_revision("rev2")

        dumped = rev.dumps()
        loaded = RevisionList.loads(dumped)
        loaded.remove_revision("rev1")
        self.assertEqual(loaded.latest_revision().revision, "rev2")

    def test_compatibility_with_timestamps(self):
        the_time = float(floor(time.time()))
        iso = from_timestamp_to_iso8601(the_time)
        old_contents = '{"revisions": [{"revision": "rev1", "time": %s}, ' \
                       '{"revision": "rev2", "time": "%s"}]}' % (the_time, the_time)
        r_list = RevisionList.loads(old_contents)
        when = r_list.get_time("rev1")
        self.assertEqual(when, iso)



import os
import unittest

from conans.test.utils.tools import TestClient


class UserChannelTestPackage(unittest.TestCase):

    def test(self):
        # https://github.com/conan-io/conan/issues/2501
        client = TestClient()
        conanfile = """from conans import ConanFile
class SayConan(ConanFile):
    pass
"""
        test = """from conans import ConanFile
class SayConan(ConanFile):
    def requirements(self):
        self.output.info("USER: %s!!" % self.user)
        self.output.info("CHANNEL: %s!!" % self.channel)

    def test(self):
        pass
"""

        client.save({"conanfile.py": conanfile,
                     "test_package/conanfile.py": test})
        client.run("create . Pkg/0.1@conan/testing")
        self.assertIn("Pkg/0.1@conan/testing (test package): USER: conan!!", client.out)
        self.assertIn("Pkg/0.1@conan/testing (test package): CHANNEL: testing!!", client.out)


class SameUserChannelTest(unittest.TestCase):

    def setUp(self):
        self.client = TestClient()
        conanfile = """
from conans import ConanFile

class SayConan(ConanFile):
    name = "Say"
    version = "0.1"
    build_policy = "missing"

    def build(self):
        self.output.info("Building %s")
"""
        for channel in ("lasote/stable", "other/testing"):
            self.client.save({"conanfile.py": conanfile % channel})
            self.client.run("export . %s" % channel)

        self.conanfile = """
from conans import ConanFile

class HelloConan(ConanFile):
    name = "Hello"
    version = "0.1"
    build_policy = "missing"

    def requirements(self):
        self.requires("Say/0.1@%s/%s" % (self.user, self.channel))

    def build(self):
        self.output.info("Building %s/%s" % (self.user, self.channel) )
"""

        self.test_conanfile = """
from conans import ConanFile, CMake
import os

class HelloReuseConan(ConanFile):
    requires = "Hello/0.1@lasote/stable"

    def test(self):
        pass
"""
        self.client.save({"conanfile.py": self.conanfile,
                          "test/conanfile.py": self.test_conanfile})

    def test_create(self):
        self.client.run("create . lasote/stable")
        self.assertIn("Say/0.1@lasote/stable: Building lasote/stable", self.client.out)
        self.assertIn("Hello/0.1@lasote/stable: Building lasote/stable", self.client.out)
        self.assertNotIn("other/testing", self.client.out)

        self.client.save({"conanfile.py": self.conanfile,
                          "test/conanfile.py": self.test_conanfile.replace("lasote/stable",
                                                                           "other/testing")})
        self.client.run("create . other/testing")
        self.assertIn("Say/0.1@other/testing: Building other/testing", self.client.out)
        self.assertIn("Hello/0.1@other/testing: Building other/testing", self.client.out)
        self.assertNotIn("lasote/stable", self.client.out)

    def test_local_commands(self):
        self.client.run("install .", assert_error=True)
        self.assertIn("ERROR: conanfile.py (Hello/0.1): "
                      "Error in requirements() method, line 10", self.client.out)
        self.assertIn("ConanException: CONAN_USERNAME environment "
                      "variable not defined, but self.user is used", self.client.out)

        os.environ["CONAN_USERNAME"] = "lasote"
        self.client.run("install .", assert_error=True)
        self.assertIn("ERROR: conanfile.py (Hello/0.1): "
                      "Error in requirements() method, line 10", self.client.out)
        self.assertIn("ConanException: CONAN_CHANNEL environment "
                      "variable not defined, but self.channel is used", self.client.out)

        os.environ["CONAN_CHANNEL"] = "stable"
        self.client.run("install .")
        self.assertIn("Say/0.1@lasote/stable: Building lasote/stable", self.client.out)
        self.assertNotIn("other/testing", self.client.out)

        os.environ["CONAN_USERNAME"] = "other"
        os.environ["CONAN_CHANNEL"] = "testing"
        self.client.run("install .")
        self.assertIn("Say/0.1@other/testing: Building other/testing", self.client.out)
        self.assertNotIn("lasote/stable", self.client.out)

        del os.environ["CONAN_USERNAME"]
        del os.environ["CONAN_CHANNEL"]

        # Now use the default_ methods to declare user and channel
        self.client = TestClient()
        conanfile = """
from conans import ConanFile

class SayConan(ConanFile):
    name = "Say"
    version = "0.1"
    build_policy = "missing"
    default_user = "userfoo"
    
    def build(self):
        self.output.info("Building %s/%s" % (self.user, self.channel) )

    @property
    def default_channel(self):
        return "channelbar"
"""
        self.client.save({"conanfile.py": conanfile})
        self.client.run("install .")
        self.client.run("build .")
        self.assertIn("Building userfoo/channelbar", self.client.out)


class BuildRequireUserChannelTest(unittest.TestCase):
    def test(self):
        # https://github.com/conan-io/conan/issues/2254
        client = TestClient()
        conanfile = """
from conans import ConanFile

class SayConan(ConanFile):
    def build_requirements(self):
        self.output.info("MYUSER: %s" % self.user)
        self.output.info("MYCHANNEL: %s" % self.channel)
"""
        client.save({"conanfile.py": conanfile})
        client.run("install . -e CONAN_USERNAME=myuser -e CONAN_CHANNEL=mychannel")
        self.assertIn("MYUSER: myuser", client.out)
        self.assertIn("MYCHANNEL: mychannel", client.out)

    def test_profile(self):
        # https://github.com/conan-io/conan/issues/2254
        client = TestClient()
        conanfile = """
from conans import ConanFile

class SayConan(ConanFile):
    def build_requirements(self):
        self.output.info("MYUSER: %s" % self.user)
        self.output.info("MYCHANNEL: %s" % self.channel)
"""
        myprofile = """[env]
CONAN_USERNAME=myuser
CONAN_CHANNEL=mychannel
"""
        client.save({"conanfile.py": conanfile,
                     "myprofile": myprofile})
        client.run("install . -pr=myprofile")
        self.assertIn("MYUSER: myuser", client.out)
        self.assertIn("MYCHANNEL: mychannel", client.out)



# coding=utf-8

import unittest

from mock import mock

from conans.client.userio import UserIO


class UserIOTest(unittest.TestCase):

    @mock.patch("conans.client.userio.UserIO.get_username", return_value="username")
    @mock.patch("conans.client.userio.UserIO.get_password", return_value="passwd")
    def test_request_login(self, m1, m2):
        user_io = UserIO()

        # Use mocked ones
        u, p = user_io.request_login(remote_name="lol")
        self.assertEqual(u, "username")
        self.assertEqual(p, "passwd")

        # Use from argument
        username = "it's me!"
        u, p = user_io.request_login(remote_name="lol", username=username)
        self.assertEqual(u, username)
        self.assertEqual(p, "passwd")



import sys
import unittest

import six

from conans.errors import ConanException
from conans.model.options import Options, OptionsValues, PackageOptionValues, PackageOptions, \
    option_undefined_msg
from conans.model.ref import ConanFileReference


class OptionsTest(unittest.TestCase):

    def setUp(self):
        package_options = PackageOptions.loads("""{static: [True, False],
        optimized: [2, 3, 4],
        path: ANY}""")
        values = PackageOptionValues()
        values.add_option("static", True)
        values.add_option("optimized", 3)
        values.add_option("path", "NOTDEF")
        package_options.values = values
        self.sut = Options(package_options)

    def test_int(self):
        self.assertEqual(3, int(self.sut.optimized))

    def test_in(self):
        package_options = PackageOptions.loads("{static: [True, False]}")
        sut = Options(package_options)
        self.assertTrue("static" in sut)
        self.assertFalse("shared" in sut)
        self.assertTrue("shared" not in sut)
        self.assertFalse("static" not in sut)

    def undefined_value_test(self):
        """ Not assigning a value to options will raise an error at validate() step
        """
        package_options = PackageOptions.loads("""{
        path: ANY}""")
        with six.assertRaisesRegex(self, ConanException, option_undefined_msg("path")):
            package_options.validate()
        package_options.path = "Something"
        package_options.validate()

    def undefined_value_none_test(self):
        """ The value None is allowed as default, not necessary to default to it
        """
        package_options = PackageOptions.loads('{path: [None, "Other"]}')
        package_options.validate()
        package_options = PackageOptions.loads('{path: ["None", "Other"]}')
        package_options.validate()

    def items_test(self):
        self.assertEqual(self.sut.items(), [("optimized", "3"), ("path", "NOTDEF"),
                                            ("static", "True")])
        self.assertEqual(self.sut.items(), [("optimized", "3"), ("path", "NOTDEF"),
                                            ("static", "True")])

    def change_test(self):
        self.sut.path = "C:/MyPath"
        self.assertEqual(self.sut.items(), [("optimized", "3"), ("path", "C:/MyPath"),
                                            ("static", "True")])
        self.assertEqual(self.sut.items(), [("optimized", "3"), ("path", "C:/MyPath"),
                                            ("static", "True")])
        with six.assertRaisesRegex(self, ConanException,
                                     "'5' is not a valid 'options.optimized' value"):
            self.sut.optimized = 5

    def boolean_test(self):
        self.sut.static = False
        self.assertFalse(self.sut.static)
        self.assertTrue(not self.sut.static)
        self.assertTrue(self.sut.static == False)
        self.assertFalse(self.sut.static == True)
        self.assertFalse(self.sut.static != False)
        self.assertTrue(self.sut.static != True)
        self.assertTrue(self.sut.static == "False")
        self.assertTrue(self.sut.static != "True")

    def basic_test(self):
        boost_values = PackageOptionValues()
        boost_values.add_option("static", False)
        boost_values.add_option("thread", True)
        boost_values.add_option("thread.multi", "off")
        poco_values = PackageOptionValues()
        poco_values.add_option("deps_bundled", True)
        hello1_values = PackageOptionValues()
        hello1_values.add_option("static", False)
        hello1_values.add_option("optimized", 4)

        options = {"Boost": boost_values,
                   "Poco": poco_values,
                   "Hello1": hello1_values}
        down_ref = ConanFileReference.loads("Hello0/0.1@diego/testing")
        own_ref = ConanFileReference.loads("Hello1/0.1@diego/testing")
        self.sut.propagate_upstream(options, down_ref, own_ref)
        self.assertEqual(self.sut.values.as_list(), [("optimized", "4"),
                                                     ("path", "NOTDEF"),
                                                     ("static", "False"),
                                                     ("Boost:static", "False"),
                                                     ("Boost:thread", "True"),
                                                     ("Boost:thread.multi", "off"),
                                                     ("Poco:deps_bundled", "True")])

        boost_values = PackageOptionValues()
        boost_values.add_option("static", 2)
        boost_values.add_option("thread", "Any")
        boost_values.add_option("thread.multi", "on")
        poco_values = PackageOptionValues()
        poco_values.add_option("deps_bundled", "What")
        hello1_values = PackageOptionValues()
        hello1_values.add_option("static", True)
        hello1_values.add_option("optimized", "2")
        options2 = {"Boost": boost_values,
                    "Poco": poco_values,
                    "Hello1": hello1_values}
        down_ref = ConanFileReference.loads("Hello2/0.1@diego/testing")

        with six.assertRaisesRegex(self, ConanException, "Hello2/0.1@diego/testing tried to change "
                                     "Hello1/0.1@diego/testing option optimized to 2"):
            self.sut.propagate_upstream(options2, down_ref, own_ref)

        self.assertEqual(self.sut.values.dumps(),
                         """optimized=4
path=NOTDEF
static=False
Boost:static=False
Boost:thread=True
Boost:thread.multi=off
Poco:deps_bundled=True""")

    def pattern_positive_test(self):
        boost_values = PackageOptionValues()
        boost_values.add_option("static", False)
        boost_values.add_option("path", "FuzzBuzz")

        options = {"Boost.*": boost_values}
        own_ref = ConanFileReference.loads("Boost.Assert/0.1@diego/testing")
        down_ref = ConanFileReference.loads("Consumer/0.1@diego/testing")
        self.sut.propagate_upstream(options, down_ref, own_ref)
        self.assertEqual(self.sut.values.as_list(), [("optimized", "3"),
                                                     ("path", "FuzzBuzz"),
                                                     ("static", "False"),
                                                     ("Boost.*:path", "FuzzBuzz"),
                                                     ("Boost.*:static", "False"),
                                                     ])

    def multi_pattern_test(self):
        boost_values = PackageOptionValues()
        boost_values.add_option("static", False)
        boost_values.add_option("path", "FuzzBuzz")
        boost_values2 = PackageOptionValues()
        boost_values2.add_option("optimized", 2)

        options = {"Boost.*": boost_values,
                   "*": boost_values2}
        own_ref = ConanFileReference.loads("Boost.Assert/0.1@diego/testing")
        down_ref = ConanFileReference.loads("Consumer/0.1@diego/testing")
        self.sut.propagate_upstream(options, down_ref, own_ref)
        self.assertEqual(self.sut.values.as_list(), [("optimized", "2"),
                                                     ("path", "FuzzBuzz"),
                                                     ("static", "False"),
                                                     ('*:optimized', '2'),
                                                     ("Boost.*:path", "FuzzBuzz"),
                                                     ("Boost.*:static", "False"),
                                                     ])

    def multi_pattern_error_test(self):
        boost_values = PackageOptionValues()
        boost_values.add_option("optimized", 4)
        boost_values2 = PackageOptionValues()
        boost_values2.add_option("optimized", 2)

        options = {"Boost.*": boost_values,
                   "*": boost_values2}
        own_ref = ConanFileReference.loads("Boost.Assert/0.1@diego/testing")
        down_ref = ConanFileReference.loads("Consumer/0.1@diego/testing")
        self.sut.propagate_upstream(options, down_ref, own_ref)
        self.assertEqual(self.sut.values.as_list(), [('optimized', '4'),
                                                     ('path', 'NOTDEF'),
                                                     ('static', 'True'),
                                                     ('*:optimized', '2'),
                                                     ('Boost.*:optimized', '4')])

    def all_positive_test(self):
        boost_values = PackageOptionValues()
        boost_values.add_option("static", False)
        boost_values.add_option("path", "FuzzBuzz")

        options = {"*": boost_values}
        own_ref = ConanFileReference.loads("Boost.Assert/0.1@diego/testing")
        down_ref = ConanFileReference.loads("Consumer/0.1@diego/testing")
        self.sut.propagate_upstream(options, down_ref, own_ref)
        self.assertEqual(self.sut.values.as_list(), [("optimized", "3"),
                                                     ("path", "FuzzBuzz"),
                                                     ("static", "False"),
                                                     ("*:path", "FuzzBuzz"),
                                                     ("*:static", "False"),
                                                     ])

    def pattern_ignore_test(self):
        boost_values = PackageOptionValues()
        boost_values.add_option("fake_option", "FuzzBuzz")

        options = {"Boost.*": boost_values}
        down_ref = ConanFileReference.loads("Consumer/0.1@diego/testing")
        own_ref = ConanFileReference.loads("Boost.Assert/0.1@diego/testing")
        self.sut.propagate_upstream(options, down_ref, own_ref)
        self.assertEqual(self.sut.values.as_list(), [("optimized", "3"),
                                                     ("path", "NOTDEF"),
                                                     ("static", "True"),
                                                     ("Boost.*:fake_option", "FuzzBuzz"),
                                                     ])

    def pattern_unmatch_test(self):
        boost_values = PackageOptionValues()
        boost_values.add_option("fake_option", "FuzzBuzz")

        options = {"OpenSSL.*": boost_values}
        down_ref = ConanFileReference.loads("Boost.Assert/0.1@diego/testing")
        own_ref = ConanFileReference.loads("Boost.Assert/0.1@diego/testing")
        self.sut.propagate_upstream(options, down_ref, own_ref)
        self.assertEqual(self.sut.values.as_list(), [("optimized", "3"),
                                                     ("path", "NOTDEF"),
                                                     ("static", "True"),
                                                     ("OpenSSL.*:fake_option", "FuzzBuzz"),
                                                     ])


class OptionsValuesPropagationUpstreamNone(unittest.TestCase):

    def test_propagate_in_options(self):
        package_options = PackageOptions.loads("""{opt: [None, "a", "b"],}""")
        values = PackageOptionValues()
        values.add_option("opt", "a")
        package_options.values = values
        sut = Options(package_options)

        other_options = PackageOptionValues()
        other_options.add_option("opt", None)
        options = {"whatever.*": other_options}
        down_ref = ConanFileReference.loads("Boost.Assert/0.1@diego/testing")
        own_ref = ConanFileReference.loads("Boost.Assert/0.1@diego/testing")
        sut.propagate_upstream(options, down_ref, own_ref)
        self.assertEqual(sut.values.as_list(), [("opt", "a"),
                                                ("whatever.*:opt", "None"),
                                                ])

    def test_propagate_in_pacakge_options(self):
        package_options = PackageOptions.loads("""{opt: [None, "a", "b"],}""")
        values = PackageOptionValues()
        package_options.values = values

        package_options.propagate_upstream({'opt': None}, None, None, [])
        self.assertEqual(package_options.values.items(), [('opt', 'None'), ])


class OptionsValuesTest(unittest.TestCase):

    def setUp(self):
        self.sut = OptionsValues.loads("""static=True
        optimized=3
        Poco:deps_bundled=True
        Boost:static=False
        Boost:thread=True
        Boost:thread.multi=off
        """)

    def test_from_list(self):
        option_values = OptionsValues(self.sut.as_list())
        self.assertEqual(option_values.dumps(), self.sut.dumps())

    def test_from_dict(self):
        options_as_dict = dict([item.split('=') for item in self.sut.dumps().splitlines()])
        option_values = OptionsValues(options_as_dict)
        self.assertEqual(option_values.dumps(), self.sut.dumps())

    def test_consistency(self):
        def _check_equal(hs1, hs2, hs3, hs4):
            opt_values1 = OptionsValues(hs1)
            opt_values2 = OptionsValues(hs2)
            opt_values3 = OptionsValues(hs3)
            opt_values4 = OptionsValues(hs4)

            self.assertEqual(opt_values1.dumps(), opt_values2.dumps())
            self.assertEqual(opt_values1.dumps(), opt_values3.dumps())
            self.assertEqual(opt_values1.dumps(), opt_values4.dumps())

        # Check that all possible input options give the same result
        _check_equal([('opt', 3)],       [('opt', '3'), ],       ('opt=3', ),       {'opt': 3})
        _check_equal([('opt', True)],    [('opt', 'True'), ],    ('opt=True', ),    {'opt': True})
        _check_equal([('opt', False)],   [('opt', 'False'), ],   ('opt=False', ),   {'opt': False})
        _check_equal([('opt', None)],    [('opt', 'None'), ],    ('opt=None', ),    {'opt': None})
        _check_equal([('opt', 0)],       [('opt', '0'), ],       ('opt=0', ),       {'opt': 0})
        _check_equal([('opt', '')],      [('opt', ''), ],        ('opt=', ),        {'opt': ''})

        # Check for leading and trailing spaces
        _check_equal([('  opt  ', 3)], [(' opt  ', '3'), ], ('  opt =3', ), {' opt ': 3})
        _check_equal([('opt', '  value  ')], [('opt', '  value '), ], ('opt= value  ', ),
                     {'opt': ' value '})

        # This is expected behaviour:
        self.assertNotEqual(OptionsValues([('opt', ''), ]).dumps(),
                            OptionsValues(('opt=""', )).dumps())

    def test_dumps(self):
        self.assertEqual(self.sut.dumps(), "\n".join(["optimized=3",
                                                      "static=True",
                                                      "Boost:static=False",
                                                      "Boost:thread=True",
                                                      "Boost:thread.multi=off",
                                                      "Poco:deps_bundled=True"]))

    def test_sha_constant(self):
        self.assertEqual(self.sut.sha,
                         "2442d43f1d558621069a15ff5968535f818939b5")
        self.sut.new_option = False
        self.sut["Boost"].new_option = "off"
        self.sut["Poco"].new_option = 0

        self.assertEqual(self.sut.dumps(), "\n".join(["new_option=False",
                                                      "optimized=3",
                                                      "static=True",
                                                      "Boost:new_option=off",
                                                      "Boost:static=False",
                                                      "Boost:thread=True",
                                                      "Boost:thread.multi=off",
                                                      "Poco:deps_bundled=True",
                                                      "Poco:new_option=0"]))
        self.assertEqual(self.sut.sha,
                         "2442d43f1d558621069a15ff5968535f818939b5")

    def test_loads_exceptions(self):
        emsg = "not enough values to unpack" if six.PY3 and sys.version_info.minor > 4 \
            else "need more than 1 value to unpack"
        with six.assertRaisesRegex(self, ValueError, emsg):
            OptionsValues.loads("a=2\nconfig\nb=3")

        with six.assertRaisesRegex(self, ValueError, emsg):
            OptionsValues.loads("config\na=2\ncommit\nb=3")

    def test_exceptions_empty_value(self):
        emsg = "not enough values to unpack" if six.PY3 and sys.version_info.minor > 4 \
            else "need more than 1 value to unpack"
        with six.assertRaisesRegex(self, ValueError, emsg):
            OptionsValues("a=2\nconfig\nb=3")

        with six.assertRaisesRegex(self, ValueError, emsg):
            OptionsValues(("a=2", "config"))

        with six.assertRaisesRegex(self, ValueError, emsg):
            OptionsValues([('a', 2), ('config', ), ])

    def test_exceptions_repeated_value(self):
        try:
            OptionsValues.loads("a=2\na=12\nb=3").dumps()
            OptionsValues(("a=2", "b=23", "a=12"))
            OptionsValues([('a', 2), ('b', True), ('a', '12')])
        except Exception as e:
            self.fail("Not expected exception: {}".format(e))

    def test_package_with_spaces(self):
        self.assertEqual(OptionsValues([('pck2:opt', 50), ]).dumps(),
                         OptionsValues([('pck2 :opt', 50), ]).dumps())



from conans.client.generators.cmake import DepsCppCmake
from conans.model import Generator


class CMakePathsGenerator(Generator):

    @property
    def filename(self):
        return "conan_paths.cmake"

    @property
    def content(self):
        lines = []
        # The CONAN_XXX_ROOT variables are needed because the FindXXX.cmake or XXXConfig.cmake
        # in a package could have been "patched" with the `cmake.patch_config_paths()`
        # replacing absolute paths with CONAN_XXX_ROOT variables.
        for dep_name, dep_cpp_info in self.deps_build_info.dependencies:
            var_name = "CONAN_{}_ROOT".format(dep_name.upper())
            lines.append('set({} {})'.format(var_name, DepsCppCmake(dep_cpp_info).rootpath))

        # We want to prioritize the FindXXX.cmake files:
        # 1. First the files found in the packages
        # 2. The previously set (by default CMAKE_MODULE_PATH is empty)
        # 3. The "install_folder" ones, in case there is no FindXXX.cmake, try with the install dir
        #    if the user used the "cmake_find_package" will find the auto-generated
        # 4. The CMake installation dir/Modules ones.
        deps = DepsCppCmake(self.deps_build_info)
        lines.append("set(CMAKE_MODULE_PATH {deps.build_paths} ${{CMAKE_MODULE_PATH}} "
                     "${{CMAKE_CURRENT_LIST_DIR}})".format(deps=deps))
        lines.append("set(CMAKE_PREFIX_PATH {deps.build_paths} ${{CMAKE_PREFIX_PATH}} "
                     "${{CMAKE_CURRENT_LIST_DIR}})".format(deps=deps))

        return "\n".join(lines)



# coding=utf-8

import textwrap
import unittest

from conans.model.ref import ConanFileReference
from conans.test.utils.tools import TestClient


class LinkedPackageAsProject(unittest.TestCase):
    conanfile_base = textwrap.dedent("""\
        from conans import ConanFile

        class APck(ConanFile):
            {body}
        """)
    conanfile = conanfile_base.format(body="pass")

    conan_package_layout = textwrap.dedent("""\
        [includedirs]
        src/include
        """)

    def setUp(self):
        self.ref_parent = ConanFileReference.loads("parent/version@user/name")
        self.ref = ConanFileReference.loads('lib/version@user/name')

        self.t = TestClient()
        self.t.save(files={'conanfile.py': self.conanfile})
        self.t.run('create . {}'.format(self.ref_parent))

        self.t.save(files={'conanfile.py':
                           self.conanfile_base.format(
                               body='requires = "{}"'.format(self.ref_parent)),
                           "mylayout": self.conan_package_layout, })
        self.t.run('editable add . {}'.format(self.ref))
        self.assertTrue(self.t.cache.installed_as_editable(self.ref))

    def tearDown(self):
        self.t.run('editable remove {}'.format(self.ref))
        self.assertFalse(self.t.cache.installed_as_editable(self.ref))


class InfoCommandOnLocalWorkspaceTest(LinkedPackageAsProject):
    """ Check that commands info/inspect running over an editable package work"""

    def test_no_args(self):
        self.t.run('info .')
        self.assertIn("conanfile.py\n"
                      "    ID: e94ed0d45e4166d2f946107eaa208d550bf3691e\n"
                      "    BuildID: None\n"
                      "    Requires:\n"
                      "        parent/version@user/name\n", self.t.out)

    def test_only_none(self):
        self.t.run('info . --only None')
        self.assertIn("parent/version@user/name\n"
                      "conanfile.py", self.t.out)

    def test_paths(self):
        self.t.run('info . --paths')
        self.assertIn("conanfile.py\n"
                      "    ID: e94ed0d45e4166d2f946107eaa208d550bf3691e\n"
                      "    BuildID: None\n"
                      "    Requires:\n"
                      "        parent/version@user/name\n", self.t.out)


class InfoCommandUsingReferenceTest(LinkedPackageAsProject):

    def test_no_args(self):
        self.t.run('info {}'.format(self.ref))
        rev = "    Revision: None\n" \
            if self.t.cache.config.revisions_enabled else ""  # Project revision is None
        expected = "lib/version@user/name\n" \
                   "    ID: e94ed0d45e4166d2f946107eaa208d550bf3691e\n" \
                   "    BuildID: None\n" \
                   "    Remote: None\n" \
                   "    Recipe: Editable\n{}" \
                   "    Binary: Editable\n" \
                   "    Binary remote: None\n" \
                   "    Requires:\n" \
                   "        parent/version@user/name\n".format(rev)
        self.assertIn(expected, self.t.out)

    def test_only_none(self):
        self.t.run('info {} --only None'.format(self.ref))
        self.assertListEqual(sorted(str(self.t.out).splitlines()),
                             sorted(["lib/version@user/name", "parent/version@user/name"]))

    def test_paths(self):
        self.t.run('info {} --paths'.format(self.ref), assert_error=True)
        self.assertIn("Operation not allowed on a package installed as editable", self.t.out)
        # TODO: Cannot show paths for a linked/editable package... what to do here?



import unittest

from conans.paths import CONANFILE
from conans.test.utils.tools import TestClient

"""

DEPENDENCY GRAPH:
-----------------

MyLib -> MyLibParent
MyLib2


BUILD DEPENDENCY GRAPH:
----------------------

BuildRequire -> BuildRequireParent (Applied both for Mylib and Mylib2 because is global)
BuildRequire2 (Applied only to MyLib2)
"""


build_require_parent = """
import os
from conans import ConanFile

class BuildRequireParent(ConanFile):
    name = "BuildRequireParent"
    version = "0.1"
    settings = "os", "compiler", "arch"

    def build(self):
        # Assert settings inherited from profile
        assert(self.settings.os == "Windows")
        assert(self.settings.compiler == "gcc")
        assert(os.environ["PROFILE_VAR_ENV"] == "PROFILE_VAR_VALUE")
        assert(os.environ["ENV_VAR_ONLY_PARENT"] == "1")

    def package_info(self):
        self.cpp_info.cflags.append("A_C_FLAG_FROM_BUILD_REQUIRE_PARENT")
        self.env_info.ENV_VAR = "ENV_VALUE_FROM_BUILD_REQUIRE_PARENT"
        self.env_info.ENV_VAR_MULTI.append("ENV_VALUE_MULTI_FROM_BUILD_REQUIRE_PARENT")
        self.cpp_info.sysroot = "path/to/folder"
"""

build_require = """
import os
from conans import ConanFile

class BuildRequire(ConanFile):
    name = "BuildRequire"
    version = "0.1"
    settings = "os", "compiler", "arch"
    requires = "BuildRequireParent/0.1@lasote/stable"
    options = {"activate_foo": [True, False]}
    default_options = "activate_foo=False"

    def build(self):
        assert(self.settings.os == "Windows")
        assert(self.settings.compiler == "gcc")
        assert(os.environ["ENV_VAR"] == "ENV_VALUE_FROM_BUILD_REQUIRE_PARENT")
        assert(os.environ["ENV_VAR_MULTI"] == "ENV_VALUE_MULTI_FROM_BUILD_REQUIRE_PARENT")
        assert(os.environ["PROFILE_VAR_ENV"] == "PROFILE_VAR_VALUE")
        assert(os.environ["ENV_VAR_ONLY_PARENT"] == "0")

    def package_info(self):
        self.cpp_info.cflags.append("A_C_FLAG_FROM_BUILD_REQUIRE")
        self.env_info.ENV_VAR = "ENV_VALUE_FROM_BUILD_REQUIRE"
        self.env_info.ENV_VAR_MULTI.append("ENV_VALUE_MULTI_FROM_BUILD_REQUIRE")
        if self.options.activate_foo:
            self.env_info.FOO_VAR = "1"
        else:
            self.env_info.FOO_VAR = "0"

"""

build_require2 = """
import os
from conans import ConanFile

class BuildRequire2(ConanFile):
    name = "BuildRequire2"
    version = "0.1"
    settings = "os", "compiler", "arch"

    def build(self):
        assert(self.settings.os == "Windows")
        assert(self.settings.compiler == "gcc")
        assert(os.environ["PROFILE_VAR_ENV"] == "PROFILE_VAR_VALUE")

    def package_info(self):
        self.cpp_info.cflags.append("A_C_FLAG_FROM_BUILD_REQUIRE2")
        self.env_info.ENV_VAR_REQ2 = "ENV_VALUE_FROM_BUILD_REQUIRE2"
        self.env_info.ENV_VAR_MULTI.append("ENV_VALUE_MULTI_FROM_BUILD_REQUIRE2")
        self.cpp_info.sysroot = "path/to/other_folder"

"""

my_lib_parent = """
import os
from conans import ConanFile

class MyLibParent(ConanFile):
    name = "MyLibParent"
    version = "0.1"
    settings = "os", "compiler", "arch"

    def build(self):
        # only from BuildRequire
        assert(os.environ["PROFILE_VAR_ENV"] == "PROFILE_VAR_VALUE")
        assert(os.environ.get("ENV_VAR", None) == "ENV_VALUE_FROM_BUILD_REQUIRE")
        assert(os.environ.get("ENV_VAR_MULTI", None) == "ENV_VALUE_MULTI_FROM_BUILD_REQUIRE"
               + os.pathsep + "ENV_VALUE_MULTI_FROM_BUILD_REQUIRE_PARENT")
        assert(os.environ.get("ENV_VAR_REQ2", None) == None)

    def package_info(self):
        self.cpp_info.cflags.append("A_C_FLAG_FROM_MYLIB_PARENT")
        self.env_info.ENV_VAR_MULTI.append("ENV_VALUE_MULTI_FROM_MYLIB_PARENT")

"""

my_lib = """
import os
from conans import ConanFile

class MyLib(ConanFile):
    name = "MyLib"
    version = "0.1"
    settings = "os", "compiler", "arch"
    requires = "MyLibParent/0.1@lasote/stable"
    generators = "cmake"

    def config_options(self):
        assert(os.environ["PROFILE_VAR_ENV"] == "PROFILE_VAR_VALUE")

    def requirements(self):
        assert(os.environ["PROFILE_VAR_ENV"] == "PROFILE_VAR_VALUE")

    def build(self):
        # only from BuildRequire
        assert(os.environ["PROFILE_VAR_ENV"] == "PROFILE_VAR_VALUE")
        assert(os.environ["ENV_VAR_ONLY_PARENT"] == "0")
        assert(os.environ["ENV_VAR"] == "ENV_VALUE_FROM_BUILD_REQUIRE")
        assert(os.environ.get("ENV_VAR_REQ2", None) == None)
        tmp = os.pathsep.join(["ENV_VALUE_MULTI_FROM_BUILD_REQUIRE",
                               "ENV_VALUE_MULTI_FROM_MYLIB_PARENT",
                               "ENV_VALUE_MULTI_FROM_BUILD_REQUIRE_PARENT"])
        assert(os.environ["ENV_VAR_MULTI"] == tmp)
        assert(self.deps_cpp_info.cflags == ["A_C_FLAG_FROM_BUILD_REQUIRE_PARENT",
                                             "A_C_FLAG_FROM_MYLIB_PARENT",
                                             "A_C_FLAG_FROM_BUILD_REQUIRE"])
        assert(self.deps_cpp_info.sysroot == "path/to/folder")

        assert(os.environ["FOO_VAR"] == "1")

"""


my_lib2 = """
import os
from conans import ConanFile

class MyLib2(ConanFile):
    name = "MyLib2"
    version = "0.1"
    settings = "os", "compiler", "arch"

    def build(self):
        # From BuildRequire and BuildRequire2
        assert(os.environ["PROFILE_VAR_ENV"] == "PROFILE_VAR_VALUE")
        assert(os.environ["ENV_VAR_ONLY_PARENT"] == "0")
        assert(os.environ["ENV_VAR"] == "ENV_VALUE_FROM_BUILD_REQUIRE")
        assert(os.environ["ENV_VAR_REQ2"] == "ENV_VALUE_FROM_BUILD_REQUIRE2")

        tmp = os.pathsep.join(["ENV_VALUE_MULTI_FROM_BUILD_REQUIRE",
                               "ENV_VALUE_MULTI_FROM_BUILD_REQUIRE2",
                               "ENV_VALUE_MULTI_FROM_BUILD_REQUIRE_PARENT"])

        assert(os.environ.get("ENV_VAR_MULTI", None) == tmp)
        assert(self.deps_cpp_info.cflags == ["A_C_FLAG_FROM_BUILD_REQUIRE_PARENT",
                                             "A_C_FLAG_FROM_BUILD_REQUIRE2",
                                             "A_C_FLAG_FROM_BUILD_REQUIRE"])

        assert(os.environ["FOO_VAR"] == "1")
        # Applied in order, so it takes the first value from BuildRequire
        # FIXME assert(self.deps_cpp_info.sysroot == "path/to/other/folder")
"""


profile = """
[build_requires]
*: BuildRequire/0.1@lasote/stable
MyLib2/*: BuildRequire2/0.1@lasote/stable

[settings]
os=Windows
compiler=gcc
compiler.version=4.8
compiler.libcxx=libstdc++
arch=x86

[env]
PROFILE_VAR_ENV=PROFILE_VAR_VALUE
BuildRequireParent:ENV_VAR_ONLY_PARENT=1
ENV_VAR_ONLY_PARENT=0

[options]
BuildRequire:activate_foo=True


"""


class ProfileRequiresTest(unittest.TestCase):

    def setUp(self):
        self.client = TestClient()

    def _export(self, var_conanfile):
        self.client.save({CONANFILE: var_conanfile}, clean_first=True)
        self.client.run("export . lasote/stable")

    def test_profile_requires(self):
        self._export(build_require_parent)
        self._export(build_require)
        self._export(build_require2)
        self._export(my_lib_parent)
        self._export(my_lib)
        self._export(my_lib2)

        reuse = """
[requires]
MyLib/0.1@lasote/stable
MyLib2/0.1@lasote/stable
[generators]
cmake
gcc
"""
        self.client.save({"profile.txt": profile, "conanfile.txt": reuse}, clean_first=True)
        self.client.run("install . --profile ./profile.txt --build missing")



import os
import unittest

from conans.test.utils.profiles import create_profile
from conans.test.utils.tools import TestClient
from conans.util.files import load
import platform


class ProfileTest(unittest.TestCase):

    def reuse_output_test(self):
        client = TestClient()
        client.run("profile new myprofile --detect")
        client.run("profile update options.Pkg:myoption=123 myprofile")
        client.run("profile update env.Pkg2:myenv=123 myprofile")
        client.run("profile show myprofile")
        self.assertIn("Pkg:myoption=123", client.out)
        self.assertIn("Pkg2:myenv=123", client.out)
        profile = str(client.out).splitlines()[2:]
        client.save({"conanfile.txt": "",
                     "mylocalprofile": "\n".join(profile)})
        client.run("install . -pr=mylocalprofile")
        self.assertIn("conanfile.txt: Generated conaninfo.txt", client.out)

    def empty_test(self):
        client = TestClient()
        client.run("profile list")
        self.assertIn("No profiles defined", client.out)

    def list_test(self):
        client = TestClient()
        profiles = ["profile1", "profile2", "profile3",
                    "nested" + os.path.sep + "profile4",
                    "nested" + os.path.sep + "two" + os.path.sep + "profile5",
                    "nested" + os.path.sep + "profile6"]
        if platform.system() != "Windows":
            profiles.append("symlink_me" + os.path.sep + "profile7")
        for profile in profiles:
            create_profile(client.cache.profiles_path, profile)

        if platform.system() != "Windows":
            os.symlink(os.path.join(client.cache.profiles_path, 'symlink_me'),
                       os.path.join(client.cache.profiles_path, 'link'))
            # profile7 will be shown twice because it is symlinked.
            profiles.append("link" + os.path.sep + "profile7")

        # Make sure local folder doesn't interact with profiles
        os.mkdir(os.path.join(client.current_folder, "profile3"))
        client.run("profile list")
        profiles.sort()
        self.assertEqual(profiles, list(str(client.out).splitlines()))

    def show_test(self):
        client = TestClient()
        create_profile(client.cache.profiles_path, "profile1", settings={"os": "Windows"},
                       options=[("MyOption", "32")])
        create_profile(client.cache.profiles_path, "profile3",
                       env=[("package:VAR", "value"), ("CXX", "/path/tomy/g++_build"),
                            ("CC", "/path/tomy/gcc_build")])
        client.run("profile show profile1")
        self.assertIn("[settings]\nos=Windows", client.out)
        self.assertIn("MyOption=32", client.out)
        client.run("profile show profile3")
        self.assertIn("CC=/path/tomy/gcc_build", client.out)
        self.assertIn("CXX=/path/tomy/g++_build", client.out)
        self.assertIn("package:VAR=value", client.out)

    def profile_update_and_get_test(self):
        client = TestClient()
        client.run("profile new ./MyProfile --detect")
        if "WARNING: GCC OLD ABI COMPATIBILITY" in client.out:
            self.assertIn("Or edit '{}/MyProfile' and "
                          "set compiler.libcxx=libstdc++11".format(client.current_folder),
                          client.out)

        pr_path = os.path.join(client.current_folder, "MyProfile")

        client.run("profile update settings.os=FakeOS ./MyProfile")
        self.assertIn("\nos=FakeOS", load(pr_path))
        self.assertNotIn("\nos=Linux", load(pr_path))

        client.run("profile get settings.os ./MyProfile")
        self.assertEqual(client.out, "FakeOS\n")

        client.run("profile update settings.compiler.version=88 ./MyProfile")
        self.assertIn("compiler.version=88", load(pr_path))

        client.run("profile get settings.compiler.version ./MyProfile")
        self.assertEqual(client.out, "88\n")

        client.run("profile update options.MyOption=23 ./MyProfile")
        self.assertIn("[options]\nMyOption=23", load(pr_path))

        client.run("profile get options.MyOption ./MyProfile")
        self.assertEqual(client.out, "23\n")

        client.run("profile update options.Package:MyOption=23 ./MyProfile")
        self.assertIn("Package:MyOption=23", load(pr_path))

        client.run("profile get options.Package:MyOption ./MyProfile")
        self.assertEqual(client.out, "23\n")

        client.run("profile update options.Package:OtherOption=23 ./MyProfile")
        self.assertIn("Package:OtherOption=23", load(pr_path))

        client.run("profile get options.Package:OtherOption ./MyProfile")
        self.assertEqual(client.out, "23\n")

        client.run("profile update env.OneMyEnv=MYVALUe ./MyProfile")
        self.assertIn("[env]\nOneMyEnv=MYVALUe", load(pr_path))

        client.run("profile get env.OneMyEnv ./MyProfile")
        self.assertEqual(client.out, "MYVALUe\n")

        # Now try the remove

        client.run("profile remove settings.os ./MyProfile")
        client.run("profile remove settings.os_build ./MyProfile")
        self.assertNotIn("os=", load(pr_path))

        client.run("profile remove settings.compiler.version ./MyProfile")
        self.assertNotIn("compiler.version", load(pr_path))

        client.run("profile remove options.MyOption ./MyProfile")
        self.assertNotIn("[options]\nMyOption", load(pr_path))

        client.run("profile remove options.Package:MyOption ./MyProfile")
        self.assertNotIn("Package:MyOption", load(pr_path))
        self.assertIn("Package:OtherOption", load(pr_path))

        client.run("profile remove env.OneMyEnv ./MyProfile")
        self.assertNotIn("OneMyEnv", load(pr_path))

        # Remove a non existent key
        client.run("profile remove settings.os ./MyProfile", assert_error=True)
        self.assertIn("Profile key 'settings.os' doesn't exist", client.out)

        client.run("profile remove options.foo ./MyProfile", assert_error=True)
        self.assertIn("Profile key 'options.foo' doesn't exist", client.out)

        client.run("profile remove env.foo ./MyProfile", assert_error=True)
        self.assertIn("Profile key 'env.foo' doesn't exist", client.out)

    def profile_update_env_test(self):
        client = TestClient()
        client.run("profile new ./MyProfile")
        pr_path = os.path.join(client.current_folder, "MyProfile")

        client.run("profile update env.foo=bar ./MyProfile")
        self.assertEqual(["[env]", "foo=bar"], load(pr_path).splitlines()[-2:])
        client.run("profile update env.foo=BAZ ./MyProfile")
        self.assertEqual(["[env]", "foo=BAZ"], load(pr_path).splitlines()[-2:])
        client.run("profile update env.MyPkg:foo=FOO ./MyProfile")
        self.assertEqual(["[env]", "foo=BAZ", "MyPkg:foo=FOO"], load(pr_path).splitlines()[-3:])
        client.run("profile update env.MyPkg:foo=FOO,BAZ,BAR ./MyProfile")
        self.assertEqual(["[env]", "foo=BAZ", "MyPkg:foo=FOO,BAZ,BAR"],
                         load(pr_path).splitlines()[-3:])

    def profile_new_test(self):
        client = TestClient()
        client.run("profile new ./MyProfile")
        pr_path = os.path.join(client.current_folder, "MyProfile")
        self.assertTrue(os.path.exists(pr_path))
        self.assertEqual(load(pr_path), """[settings]
[options]
[build_requires]
[env]
""")

        client.run("profile new ./MyProfile2 --detect")
        pr_path = os.path.join(client.current_folder, "MyProfile2")
        self.assertTrue(os.path.exists(os.path.join(client.current_folder, "MyProfile2")))
        self.assertIn("os=", load(pr_path))

        client.run("profile new ./MyProfile2 --detect", assert_error=True)
        self.assertIn("Profile already exists", client.out)

        client.run("profile new MyProfile3")
        pr_path = os.path.join(client.cache.profiles_path, "MyProfile3")
        self.assertTrue(os.path.exists(pr_path))
        self.assertNotIn("os=", load(pr_path))

    def profile_force_new_test(self):
        client = TestClient()

        empty_profile = """[settings]
[options]
[build_requires]
[env]
"""
        client.run("profile new ./MyProfile")
        pr_path = os.path.join(client.current_folder, "MyProfile")
        self.assertTrue(os.path.exists(pr_path))
        self.assertEqual(load(pr_path), empty_profile)

        client.run("profile new ./MyProfile --detect", assert_error=True)
        self.assertIn("Profile already exists", client.out)

        client.run("profile new ./MyProfile --detect --force", assert_error=False)
        self.assertNotEqual(load(pr_path), empty_profile)

        detected_profile = load(pr_path)

        client.run("profile update settings.os=FakeOS ./MyProfile")
        self.assertIn("\nos=FakeOS", load(pr_path))

        client.run("profile update env.MyEnv=MYVALUe ./MyProfile")
        self.assertIn("[env]\nMyEnv=MYVALUe", load(pr_path))

        client.run("profile new ./MyProfile --detect --force", assert_error=False)
        self.assertNotIn("\nos=FakeOS", load(pr_path))
        self.assertNotIn("[env]\nMyEnv=MYVALUe", load(pr_path))
        self.assertEqual(load(pr_path), detected_profile)

    def missing_subarguments_test(self):
        client = TestClient()
        client.run("profile", assert_error=True)
        self.assertIn("ERROR: Exiting with code: 2", client.out)



# coding=utf-8
import os
import six
import unittest
import subprocess

from mock import patch
from nose.plugins.attrib import attr
from parameterized import parameterized

from conans.client import tools
from conans.client.tools.scm import Git
from conans.errors import ConanException
from conans.test.utils.tools import temp_folder, create_local_git_repo,\
    TestClient
from conans.util.files import save


class GitRemoteUrlTest(unittest.TestCase):

    def test_remove_credentials(self):
        """ Check that the 'remove_credentials' argument is taken into account """
        expected_url = 'https://myrepo.com/path/to/repo.git'
        origin_url = 'https://username:password@myrepo.com/path/to/repo.git'

        git = Git(folder=temp_folder())
        git.run("init .")
        git.run("remote add origin {}".format(origin_url))

        self.assertEqual(git.get_remote_url(), origin_url)
        self.assertEqual(git.get_remote_url(remove_credentials=True), expected_url)


@attr('git')
class GitToolTest(unittest.TestCase):

    @patch('subprocess.Popen')
    def test_version(self, mocked_open):
        mocked_open.return_value.communicate.return_value = ('git version 2.21.0'.encode(), None)
        version = Git.get_version()
        self.assertEqual(version, "2.21.0")

    @patch('subprocess.Popen')
    def test_version_invalid(self, mocked_open):
        mocked_open.return_value.communicate.return_value = ('failed'.encode(), None)
        with self.assertRaises(ConanException):
            Git.get_version()

    def test_repo_root(self):
        root_path, _ = create_local_git_repo({"myfile": "anything"})

        # Initialized in the root folder
        git = Git(root_path)
        self.assertEqual(root_path, git.get_repo_root())

        # Initialized elsewhere
        subfolder = os.path.join(root_path, 'subfolder')
        os.makedirs(subfolder)
        git = Git(subfolder)
        self.assertEqual(root_path, git.get_repo_root())

    def test_is_pristine(self):
        root_path, _ = create_local_git_repo({"myfile": "anything"})

        git = Git(root_path)
        self.assertTrue(git.is_pristine())

        save(os.path.join(root_path, "other_file"), "content")
        self.assertFalse(git.is_pristine())

        git.run("add .")
        self.assertFalse(git.is_pristine())

        git.run('commit -m "commit"')
        self.assertTrue(git.is_pristine())

    def test_is_local_repository(self):
        root_path, _ = create_local_git_repo({"myfile": "anything"})

        git = Git(temp_folder())
        git.clone(root_path)
        self.assertTrue(git.is_local_repository())
        # TODO: Check that with remote one it is working too

    def test_clone_git(self):
        path, _ = create_local_git_repo({"myfile": "contents"})
        tmp = temp_folder()
        git = Git(tmp)
        git.clone(path)
        self.assertTrue(os.path.exists(os.path.join(tmp, "myfile")))

    @parameterized.expand([(None,),  # default
                           ("develop",),  # branch name
                           ("1.0",),  # tag name
                           ("HEAD",),  # expression
                           ])
    def test_clone_git_shallow(self, element):
        path, revision = create_local_git_repo({"myfile": "contents"}, commits=3, tags=["1.0"], branch="develop")
        tmp = temp_folder()
        git = Git(tmp)
        git.clone("file://" + path, branch=element, shallow=True)  # --depth is ignored in local clones
        with self.assertRaises(subprocess.CalledProcessError):
            git.checkout(element="HEAD~1")
        self.assertTrue(os.path.exists(os.path.join(tmp, "myfile")))
        self.assertEqual(git.get_revision(), revision)
        self.assertEqual(git.run("rev-list --all --count"), "1")

    def test_clone_git_shallow_revision(self):
        path, revision = create_local_git_repo({"myfile": "contents"}, commits=3, tags=["1.0"], branch="develop")
        tmp = temp_folder()
        git = Git(tmp)
        if Git.get_version() < "2.13":
            # older Git versions have known bugs with "git fetch origin <sha>":
            # https://github.com/git/git/blob/master/Documentation/RelNotes/2.13.0.txt
            #  * "git fetch" that requests a commit by object name, when the other
            #    side does not allow such an request, failed without much
            #    explanation.
            # https://github.com/git/git/blob/master/Documentation/RelNotes/2.14.0.txt
            # * There is no good reason why "git fetch $there $sha1" should fail
            #    when the $sha1 names an object at the tip of an advertised ref,
            #    even when the other side hasn't enabled allowTipSHA1InWant.
            with self.assertRaises(subprocess.CalledProcessError):
                git.clone("file://" + path, branch=revision, shallow=True)
        else:
            git.clone("file://" + path, branch=revision, shallow=True)
            with self.assertRaises(subprocess.CalledProcessError):
                git.checkout(element="HEAD~1")
            self.assertTrue(os.path.exists(os.path.join(tmp, "myfile")))
            self.assertEqual(git.get_revision(), revision)
            self.assertEqual(git.run("rev-list --all --count"), "1")

    def test_clone_git_shallow_with_local(self):
        path, revision = create_local_git_repo({"repofile": "contents"}, commits=3)
        tmp = temp_folder()
        save(os.path.join(tmp, "localfile"), "contents")
        save(os.path.join(tmp, "indexfile"), "contents")
        git = Git(tmp)
        git.run("init")
        git.run("add indexfile")
        git.clone("file://" + path, branch="master", shallow=True)  # --depth is ignored in local clones
        self.assertTrue(os.path.exists(os.path.join(tmp, "repofile")))
        self.assertTrue(os.path.exists(os.path.join(tmp, "localfile")))
        self.assertTrue(os.path.exists(os.path.join(tmp, "indexfile")))
        self.assertEqual(git.get_revision(), revision)
        self.assertEqual(git.run("rev-list --all --count"), "1")

    def test_clone_existing_folder_git(self):
        path, commit = create_local_git_repo({"myfile": "contents"}, branch="my_release")

        tmp = temp_folder()
        save(os.path.join(tmp, "file"), "dummy contents")
        git = Git(tmp)
        git.clone(path, branch="my_release")
        self.assertTrue(os.path.exists(os.path.join(tmp, "myfile")))

        # Checkout a commit
        git.checkout(commit)
        self.assertEqual(git.get_revision(), commit)

    def test_clone_existing_folder_without_branch(self):
        tmp = temp_folder()
        save(os.path.join(tmp, "file"), "dummy contents")
        git = Git(tmp)
        with six.assertRaisesRegex(self, ConanException, "specify a branch to checkout"):
            git.clone("https://github.com/conan-community/conan-zlib.git")

    def test_credentials(self):
        tmp = temp_folder()
        git = Git(tmp, username="peter", password="otool")
        url_credentials = git.get_url_with_credentials("https://some.url.com")
        self.assertEqual(url_credentials, "https://peter:otool@some.url.com")

    def test_verify_ssl(self):
        class MyRunner(object):
            def __init__(self):
                self.calls = []

            def __call__(self, *args, **kwargs):
                self.calls.append(args[0])
                return ""

        runner = MyRunner()
        tmp = temp_folder()
        git = Git(tmp, username="peter", password="otool", verify_ssl=True, runner=runner,
                  force_english=True)
        git.clone(url="https://myrepo.git")
        self.assertIn("git -c http.sslVerify=true", runner.calls[0])

        runner = MyRunner()
        git = Git(tmp, username="peter", password="otool", verify_ssl=False, runner=runner,
                  force_english=False)
        git.clone(url="https://myrepo.git")
        self.assertIn("git -c http.sslVerify=false", runner.calls[0])

    def test_clone_submodule_git(self):
        subsubmodule, _ = create_local_git_repo({"subsubmodule": "contents"})
        submodule, _ = create_local_git_repo({"submodule": "contents"}, submodules=[subsubmodule])
        path, commit = create_local_git_repo({"myfile": "contents"}, submodules=[submodule])

        def _create_paths():
            tmp = temp_folder()
            submodule_path = os.path.join(
                tmp,
                os.path.basename(os.path.normpath(submodule)))
            subsubmodule_path = os.path.join(
                submodule_path,
                os.path.basename(os.path.normpath(subsubmodule)))
            return tmp, submodule_path, subsubmodule_path

        # Check old (default) behaviour
        tmp, submodule_path, _ = _create_paths()
        git = Git(tmp)
        git.clone(path)
        self.assertTrue(os.path.exists(os.path.join(tmp, "myfile")))
        self.assertFalse(os.path.exists(os.path.join(submodule_path, "submodule")))

        # Check invalid value
        tmp, submodule_path, _ = _create_paths()
        git = Git(tmp)
        git.clone(path)
        with six.assertRaisesRegex(self, ConanException,
                                   "Invalid 'submodule' attribute value in the 'scm'."):
            git.checkout(commit, submodule="invalid")

        # Check shallow
        tmp, submodule_path, subsubmodule_path = _create_paths()
        git = Git(tmp)
        git.clone(path)
        git.checkout(commit, submodule="shallow")
        self.assertTrue(os.path.exists(os.path.join(tmp, "myfile")))
        self.assertTrue(os.path.exists(os.path.join(submodule_path, "submodule")))
        self.assertFalse(os.path.exists(os.path.join(subsubmodule_path, "subsubmodule")))

        # Check recursive
        tmp, submodule_path, subsubmodule_path = _create_paths()
        git = Git(tmp)
        git.clone(path)
        git.checkout(commit, submodule="recursive")
        self.assertTrue(os.path.exists(os.path.join(tmp, "myfile")))
        self.assertTrue(os.path.exists(os.path.join(submodule_path, "submodule")))
        self.assertTrue(os.path.exists(os.path.join(subsubmodule_path, "subsubmodule")))

    def git_to_capture_branch_test(self):
        conanfile = """
import re
from conans import ConanFile, tools

def get_version():
    git = tools.Git()
    try:
        branch = git.get_branch()
        branch = re.sub('[^0-9a-zA-Z]+', '_', branch)
        return "%s_%s" % (branch, git.get_revision())
    except:
        return None

class HelloConan(ConanFile):
    name = "Hello"
    version = get_version()

    def build(self):
        assert("r3le_ase__" in self.version)
        assert(len(self.version) == 50)
"""
        path, _ = create_local_git_repo({"conanfile.py": conanfile}, branch="r3le-ase-")
        client = TestClient()
        client.current_folder = path
        client.run("create . user/channel")

    def git_helper_in_recipe_test(self):
        client = TestClient()
        git_repo = temp_folder()
        save(os.path.join(git_repo, "file.h"), "contents")
        with client.chdir(git_repo):
            client.run_command("git init .")
            client.run_command('git config user.email "you@example.com"')
            client.run_command('git config user.name "Your Name"')
            client.run_command("git checkout -b dev")
            client.run_command("git add .")
            client.run_command('git commit -m "comm"')

        conanfile = """
import os
from conans import ConanFile, tools

class HelloConan(ConanFile):
    name = "Hello"
    version = "0.1"
    exports_sources = "other"

    def source(self):
        git = tools.Git()
        git.clone("%s", "dev")

    def build(self):
        assert(os.path.exists("file.h"))
""" % git_repo.replace("\\", "/")
        client.save({"conanfile.py": conanfile, "other": "hello"})
        client.run("create . user/channel")

        # Now clone in a subfolder with later checkout
        conanfile = """
import os
from conans import ConanFile, tools

class HelloConan(ConanFile):
    name = "Hello"
    version = "0.1"
    exports_sources = "other"

    def source(self):
        tools.mkdir("src")
        git = tools.Git("./src")
        git.clone("%s")
        git.checkout("dev")

    def build(self):
        assert(os.path.exists(os.path.join("src", "file.h")))
""" % git_repo.replace("\\", "/")
        client.save({"conanfile.py": conanfile, "other": "hello"})
        client.run("create . user/channel")

        # Base dir, with exports without subfolder and not specifying checkout fails
        conanfile = """
import os
from conans import ConanFile, tools

class HelloConan(ConanFile):
    name = "Hello"
    version = "0.1"
    exports_sources = "other"

    def source(self):
        git = tools.Git()
        git.clone("%s")

    def build(self):
        assert(os.path.exists("file.h"))
""" % git_repo.replace("\\", "/")
        client.save({"conanfile.py": conanfile, "other": "hello"})
        client.run("create . user/channel", assert_error=True)
        self.assertIn("specify a branch to checkout", client.out)

    def git_commit_message_test(self):
        client = TestClient()
        git_repo = temp_folder()
        with client.chdir(git_repo):
            client.run_command("git init .")
            client.run_command('git config user.email "you@example.com"')
            client.run_command('git config user.name "Your Name"')
            client.run_command("git checkout -b dev")
        git = Git(git_repo)
        self.assertIsNone(git.get_commit_message())
        save(os.path.join(git_repo, "test"), "contents")
        with client.chdir(git_repo):
            client.run_command("git add test")
            client.run_command('git commit -m "first commit"')
        self.assertEqual("dev", git.get_branch())
        self.assertEqual("first commit", git.get_commit_message())


class GitToolsTests(unittest.TestCase):

    def setUp(self):
        self.folder, self.rev = create_local_git_repo({'myfile.txt': "contents"})

    def test_no_tag(self):
        """
        No tags has been created in repo
        """
        git = Git(folder=self.folder)
        tag = git.get_tag()
        self.assertIsNone(tag)

    def test_in_tag(self):
        """
        Current checkout is on a tag
        """
        git = Git(folder=self.folder)
        git.run("tag 0.0.0")
        tag = git.get_tag()
        self.assertEqual("0.0.0", tag)

    def test_in_branch_with_tag(self):
        """
        Tag is defined but current commit is ahead of it
        """
        git = Git(folder=self.folder)
        git.run("tag 0.0.0")
        save(os.path.join(self.folder, "file.txt"), "")
        git.run("add .")
        git.run("commit -m \"new file\"")
        tag = git.get_tag()
        self.assertIsNone(tag)

    def test_get_tag_no_git_repo(self):
        """
        Try to get tag out of a git repo
        """
        git = Git(folder=temp_folder())
        with six.assertRaisesRegex(self, ConanException, "Not a valid 'git' repository"):
            git.get_tag()

    def test_excluded_files(self):
        folder = temp_folder()
        save(os.path.join(folder, "file"), "some contents")
        git = Git(folder)
        with tools.environment_append({"PATH": ""}):
            git.excluded_files()



import os
import stat
import textwrap
import unittest

from parameterized import parameterized

from conans.model.manifest import FileTreeManifest
from conans.model.ref import ConanFileReference
from conans.paths import CONANFILE, CONAN_MANIFEST
from conans.test.utils.cpp_test_files import cpp_hello_conan_files
from conans.test.utils.tools import TestClient
from conans.test.utils.tools import create_local_git_repo
from conans.util.files import load, save


class ExportSettingsTest(unittest.TestCase):

    def test_basic(self):
        client = TestClient()
        conanfile = """
from conans import ConanFile
class TestConan(ConanFile):
    name = "Hello"
    version = "1.2"
    settings = {"os": ["Linux"]}
"""
        files = {CONANFILE: conanfile}
        client.save(files)
        client.run("export . lasote/stable")
        self.assertIn("WARN: Conanfile doesn't have 'license'", client.out)
        client.run("install Hello/1.2@lasote/stable -s os=Windows", assert_error=True)
        self.assertIn("'Windows' is not a valid 'settings.os' value", client.out)
        self.assertIn("Possible values are ['Linux']", client.out)

    def export_without_full_reference_test(self):
        client = TestClient()
        client.save({"conanfile.py": """from conans import ConanFile
class MyPkg(ConanFile):
    pass
"""})
        client.run("export . lasote/stable", assert_error=True)
        self.assertIn("conanfile didn't specify name", client.out)

        client.save({"conanfile.py": """from conans import ConanFile
class MyPkg(ConanFile):
    name="Lib"
"""})
        client.run("export . lasote/stable", assert_error=True)
        self.assertIn("conanfile didn't specify version", client.out)

        client.save({"conanfile.py": """from conans import ConanFile
class MyPkg(ConanFile):
    pass
"""})
        client.run("export . lib/1.0@lasote/channel")
        self.assertIn("lib/1.0@lasote/channel: A new conanfile.py version was exported",
                      client.out)

        client.save({"conanfile.py": """from conans import ConanFile
class MyPkg(ConanFile):
    name="Lib"
    version="1.0"
"""})
        client.run("export . lasote", assert_error=True)
        self.assertIn("Invalid parameter 'lasote', specify the full reference or user/channel",
                      client.out)

    def test_export_read_only(self):
        client = TestClient()
        conanfile = """
from conans import ConanFile
class TestConan(ConanFile):
    name = "Hello"
    version = "1.2"
    exports = "file1.txt"
    exports_sources = "file2.txt"
"""
        ref = ConanFileReference.loads("Hello/1.2@lasote/stable")
        export_path = client.cache.package_layout(ref).export()
        export_src_path = client.cache.package_layout(ref).export_sources()

        files = {CONANFILE: conanfile,
                 "file1.txt": "",
                 "file2.txt": ""}
        client.save(files)
        mode1 = os.stat(os.path.join(client.current_folder, "file1.txt")).st_mode
        mode2 = os.stat(os.path.join(client.current_folder, "file2.txt")).st_mode
        os.chmod(os.path.join(client.current_folder, "file1.txt"), mode1 &~ stat.S_IWRITE)
        os.chmod(os.path.join(client.current_folder, "file2.txt"), mode2 &~ stat.S_IWRITE)

        client.run("export . lasote/stable")
        self.assertEqual(load(os.path.join(export_path, "file1.txt")), "")
        self.assertEqual(load(os.path.join(export_src_path, "file2.txt")), "")
        with self.assertRaises(IOError):
            save(os.path.join(export_path, "file1.txt"), "")
        with self.assertRaises(IOError):
            save(os.path.join(export_src_path, "file2.txt"), "")
        self.assertIn("WARN: Conanfile doesn't have 'license'", client.out)
        files = {CONANFILE: conanfile,
                 "file1.txt": "file1",
                 "file2.txt": "file2"}
        os.chmod(os.path.join(client.current_folder, "file1.txt"), mode1 | stat.S_IWRITE)
        os.chmod(os.path.join(client.current_folder, "file2.txt"), mode2 | stat.S_IWRITE)
        client.save(files)
        client.run("export . lasote/stable")

        self.assertEqual(load(os.path.join(export_path, "file1.txt")), "file1")
        self.assertEqual(load(os.path.join(export_src_path, "file2.txt")), "file2")
        client.run("install Hello/1.2@lasote/stable --build=missing")
        self.assertIn("Hello/1.2@lasote/stable: Generating the package", client.out)

        files = {CONANFILE: conanfile,
                 "file1.txt": "",
                 "file2.txt": ""}
        client.save(files)
        os.chmod(os.path.join(client.current_folder, "file1.txt"), mode1 &~ stat.S_IWRITE)
        os.chmod(os.path.join(client.current_folder, "file2.txt"), mode2 &~ stat.S_IWRITE)
        client.run("export . lasote/stable")
        self.assertEqual(load(os.path.join(export_path, "file1.txt")), "")
        self.assertEqual(load(os.path.join(export_src_path, "file2.txt")), "")
        client.run("install Hello/1.2@lasote/stable --build=Hello")
        self.assertIn("Hello/1.2@lasote/stable: Generating the package", client.out)

    def test_code_parent(self):
        """ when referencing the parent, the relative folder "sibling" will be kept
        """
        base = """
from conans import ConanFile
class TestConan(ConanFile):
    name = "Hello"
    version = "1.2"
    exports = "../*.txt"
"""
        for conanfile in (base, base.replace("../*.txt", "../sibling*")):
            client = TestClient()
            files = {"recipe/conanfile.py": conanfile,
                     "sibling/file.txt": "Hello World!"}
            client.save(files)
            client.current_folder = os.path.join(client.current_folder, "recipe")
            client.run("export . lasote/stable")
            ref = ConanFileReference("Hello", "1.2", "lasote", "stable")
            export_path = client.cache.package_layout(ref).export()
            content = load(os.path.join(export_path, "sibling/file.txt"))
            self.assertEqual("Hello World!", content)

    def test_code_sibling(self):
        # if provided a path with slash, it will use as a export base
        client = TestClient()
        conanfile = """
from conans import ConanFile
class TestConan(ConanFile):
    name = "Hello"
    version = "1.2"
    exports = "../sibling/*.txt"
"""
        files = {"recipe/conanfile.py": conanfile,
                 "sibling/file.txt": "Hello World!"}
        client.save(files)
        client.current_folder = os.path.join(client.current_folder, "recipe")
        client.run("export . lasote/stable")
        ref = ConanFileReference("Hello", "1.2", "lasote", "stable")
        export_path = client.cache.package_layout(ref).export()
        content = load(os.path.join(export_path, "file.txt"))
        self.assertEqual("Hello World!", content)

    def test_code_several_sibling(self):
        # if provided a path with slash, it will use as a export base
        client = TestClient()
        conanfile = """
from conans import ConanFile
class TestConan(ConanFile):
    name = "Hello"
    version = "1.2"
    exports_sources = "../test/src/*", "../cpp/*", "../include/*"
"""
        files = {"recipe/conanfile.py": conanfile,
                 "test/src/file.txt": "Hello World!",
                 "cpp/file.cpp": "Hello World!",
                 "include/file.h": "Hello World!"}
        client.save(files)
        client.current_folder = os.path.join(client.current_folder, "recipe")
        client.run("export . lasote/stable")
        ref = ConanFileReference("Hello", "1.2", "lasote", "stable")
        export_path = client.cache.package_layout(ref).export_sources()
        self.assertEqual(sorted(['file.txt', 'file.cpp', 'file.h']),
                         sorted(os.listdir(export_path)))

    @parameterized.expand([("myconanfile.py", ), ("Conanfile.py", )])
    def test_filename(self, filename):
        client = TestClient()
        conanfile = """
from conans import ConanFile
class TestConan(ConanFile):
    name = "Hello"
    version = "1.2"
"""

        client.save({filename: conanfile})
        client.run("export %s lasote/stable" % filename)
        self.assertIn("Hello/1.2@lasote/stable: A new conanfile.py version was exported",
                      client.out)
        ref = ConanFileReference("Hello", "1.2", "lasote", "stable")
        export_path = client.cache.package_layout(ref).export()
        conanfile = load(os.path.join(export_path, "conanfile.py"))
        self.assertIn('name = "Hello"', conanfile)
        manifest = load(os.path.join(export_path, "conanmanifest.txt"))
        self.assertIn('conanfile.py: cac514c81a0af0d87fa379b0bf16fbaa', manifest)

    def test_exclude_basic(self):
        client = TestClient()
        conanfile = """
from conans import ConanFile
class TestConan(ConanFile):
    name = "Hello"
    version = "1.2"
    exports = "*.txt", "!*file1.txt"
    exports_sources = "*.cpp", "!*temp.cpp"
"""

        client.save({CONANFILE: conanfile,
                     "file.txt": "",
                     "file1.txt": "",
                     "file.cpp": "",
                     "file_temp.cpp": ""})
        client.run("export . lasote/stable")
        ref = ConanFileReference("Hello", "1.2", "lasote", "stable")
        export_path = client.cache.package_layout(ref).export()
        exports_sources_path = client.cache.package_layout(ref).export_sources()
        self.assertTrue(os.path.exists(os.path.join(export_path, "file.txt")))
        self.assertFalse(os.path.exists(os.path.join(export_path, "file1.txt")))
        self.assertTrue(os.path.exists(os.path.join(exports_sources_path, "file.cpp")))
        self.assertFalse(os.path.exists(os.path.join(exports_sources_path, "file_temp.cpp")))

    def test_exclude_folders(self):
        client = TestClient()
        conanfile = """
from conans import ConanFile
class TestConan(ConanFile):
    name = "Hello"
    version = "1.2"
    exports = "*.txt", "!*/temp/*"
"""

        client.save({CONANFILE: conanfile,
                     "file.txt": "",
                     "any/temp/file1.txt": "",
                     "other/sub/file2.txt": ""})
        client.run("export . lasote/stable")
        ref = ConanFileReference("Hello", "1.2", "lasote", "stable")
        export_path = client.cache.package_layout(ref).export()
        self.assertTrue(os.path.exists(os.path.join(export_path, "file.txt")))
        self.assertFalse(os.path.exists(os.path.join(export_path, "any/temp/file1.txt")))
        self.assertTrue(os.path.exists(os.path.join(export_path, "other/sub/file2.txt")))


class ExportTest(unittest.TestCase):

    def setUp(self):
        self.client = TestClient()
        self.files = cpp_hello_conan_files("Hello0", "0.1")
        self.ref = ConanFileReference("Hello0", "0.1", "lasote", "stable")
        self.client.save(self.files)
        self.client.run("export . lasote/stable")

    def test_basic(self):
        """ simple registration of a new conans
        """
        reg_path = self.client.cache.package_layout(self.ref).export()
        manif = FileTreeManifest.load(self.client.cache.package_layout(self.ref).export())

        self.assertIn('%s: A new conanfile.py version was exported' % str(self.ref),
                      self.client.out)
        self.assertIn('%s: Folder: %s' % (str(self.ref), reg_path), self.client.out)
        self.assertTrue(os.path.exists(reg_path))

        for name in list(self.files.keys()):
            self.assertTrue(os.path.exists(os.path.join(reg_path, name)))

        expected_sums = {'hello.cpp': '4f005274b2fdb25e6113b69774dac184',
                         'main.cpp': '0479f3c223c9a656a718f3148e044124',
                         'CMakeLists.txt': '10d907c160c360b28f6991397a5aa9b4',
                         'conanfile.py': '355949fbf0b4fc32b8f1c5a338dfe1ae',
                         'executable': '68b329da9893e34099c7d8ad5cb9c940',
                         'helloHello0.h': '9448df034392fc8781a47dd03ae71bdd'}
        self.assertEqual(expected_sums, manif.file_sums)

    def test_case_sensitive(self):
        self.files = cpp_hello_conan_files("hello0", "0.1")
        self.ref = ConanFileReference("hello0", "0.1", "lasote", "stable")
        self.client.save(self.files)
        self.client.run("export . lasote/stable", assert_error=True)
        self.assertIn("ERROR: Cannot export package with same name but different case",
                      self.client.out)

    def test_export_filter(self):
        content = """
from conans import ConanFile

class OpenSSLConan(ConanFile):
    name = "openssl"
    version = "2.0.1"
"""
        save(os.path.join(self.client.current_folder, CONANFILE), content)
        self.client.run("export . lasote/stable")
        ref = ConanFileReference.loads('openssl/2.0.1@lasote/stable')
        reg_path = self.client.cache.package_layout(ref).export()
        self.assertEqual(sorted(os.listdir(reg_path)),
                         [CONANFILE, CONAN_MANIFEST])

        content = """
from conans import ConanFile

class OpenSSLConan(ConanFile):
    name = "openssl"
    version = "2.0.1"
    exports = ('*.txt', '*.h')
"""
        save(os.path.join(self.client.current_folder, CONANFILE), content)
        self.client.run("export . lasote/stable")
        self.assertEqual(sorted(os.listdir(reg_path)),
                         ['CMakeLists.txt', CONANFILE, CONAN_MANIFEST,
                          'helloHello0.h'])

        # Now exports being a list instead a tuple
        content = """
from conans import ConanFile

class OpenSSLConan(ConanFile):
    name = "openssl"
    version = "2.0.1"
    exports = ['*.txt', '*.h']
"""
        save(os.path.join(self.client.current_folder, CONANFILE), content)
        self.client.run("export . lasote/stable")
        self.assertEqual(sorted(os.listdir(reg_path)),
                         ['CMakeLists.txt', CONANFILE, CONAN_MANIFEST, 'helloHello0.h'])

    def test_export_the_same_code(self):
        file_list = self._create_packages_and_builds()
        # Export the same conans
        # Do not adjust cpu_count, it is reusing a cache
        client2 = TestClient(self.client.cache_folder, cpu_count=False)
        files2 = cpp_hello_conan_files("Hello0", "0.1")
        client2.save(files2)
        client2.run("export . lasote/stable")
        reg_path2 = client2.cache.package_layout(self.ref).export()
        digest2 = FileTreeManifest.load(client2.cache.package_layout(self.ref).export())

        self.assertNotIn('A new Conan version was exported', client2.out)
        self.assertNotIn('Cleaning the old builds ...', client2.out)
        self.assertNotIn('Cleaning the old packs ...', client2.out)
        self.assertNotIn('All the previous packs were cleaned', client2.out)
        self.assertIn('%s: A new conanfile.py version was exported' % str(self.ref),
                      self.client.out)
        self.assertIn('%s: Folder: %s' % (str(self.ref), reg_path2), self.client.out)
        self.assertTrue(os.path.exists(reg_path2))

        for name in list(files2.keys()):
            self.assertTrue(os.path.exists(os.path.join(reg_path2, name)))

        expected_sums = {'hello.cpp': '4f005274b2fdb25e6113b69774dac184',
                         'main.cpp': '0479f3c223c9a656a718f3148e044124',
                         'CMakeLists.txt': '10d907c160c360b28f6991397a5aa9b4',
                         'conanfile.py': '355949fbf0b4fc32b8f1c5a338dfe1ae',
                         'executable': '68b329da9893e34099c7d8ad5cb9c940',
                         'helloHello0.h': '9448df034392fc8781a47dd03ae71bdd'}
        self.assertEqual(expected_sums, digest2.file_sums)

        for f in file_list:
            self.assertTrue(os.path.exists(f))

    def test_export_a_new_version(self):
        self._create_packages_and_builds()
        # Export an update of the same conans

        # Do not adjust cpu_count, it is reusing a cache
        client2 = TestClient(self.client.cache_folder, cpu_count=False)
        files2 = cpp_hello_conan_files("Hello0", "0.1")
        files2[CONANFILE] = "# insert comment\n %s" % files2[CONANFILE]
        client2.save(files2)
        client2.run("export . lasote/stable")

        reg_path3 = client2.cache.package_layout(self.ref).export()
        digest3 = FileTreeManifest.load(client2.cache.package_layout(self.ref).export())

        self.assertIn('%s: A new conanfile.py version was exported' % str(self.ref),
                      self.client.out)
        self.assertIn('%s: Folder: %s' % (str(self.ref), reg_path3), self.client.out)

        self.assertTrue(os.path.exists(reg_path3))

        for name in list(files2.keys()):
            self.assertTrue(os.path.exists(os.path.join(reg_path3, name)))

        expected_sums = {'hello.cpp': '4f005274b2fdb25e6113b69774dac184',
                         'main.cpp': '0479f3c223c9a656a718f3148e044124',
                         'CMakeLists.txt': '10d907c160c360b28f6991397a5aa9b4',
                         'conanfile.py': 'ad17cf00b3142728b03ac37782b9acd9',
                         'executable': '68b329da9893e34099c7d8ad5cb9c940',
                         'helloHello0.h': '9448df034392fc8781a47dd03ae71bdd'}
        self.assertEqual(expected_sums, digest3.file_sums)

        # for f in file_list:
        #    self.assertFalse(os.path.exists(f))

    def _create_packages_and_builds(self):
        reg_builds = self.client.cache.package_layout(self.ref).builds()
        reg_packs = self.client.cache.package_layout(self.ref).packages()

        folders = [os.path.join(reg_builds, '342525g4f52f35f'),
                   os.path.join(reg_builds, 'ew9o8asdf908asdf80'),
                   os.path.join(reg_packs, '342525g4f52f35f'),
                   os.path.join(reg_packs, 'ew9o8asdf908asdf80')]

        file_list = []
        for f in folders:
            for name, content in {'file1.h': 'asddfasdf', 'file1.dll': 'asddfasdf'}.items():
                file_path = os.path.join(f, name)
                save(file_path, content)
                file_list.append(file_path)
        return file_list


class ExportMetadataTest(unittest.TestCase):
    conanfile = textwrap.dedent("""
        from conans import ConanFile

        class Lib(ConanFile):
            revision_mode = "{revision_mode}"
    """)

    summary_hash = "bfe8b4a6a2a74966c0c4e0b34705004a"

    def test_revision_mode_hash(self):
        t = TestClient()
        t.save({'conanfile.py': self.conanfile.format(revision_mode="hash")})

        ref = ConanFileReference.loads("name/version@user/channel")
        t.run("export . {}".format(ref))

        meta = t.cache.package_layout(ref, short_paths=False).load_metadata()
        self.assertEqual(meta.recipe.revision, self.summary_hash)

    def test_revision_mode_scm(self):
        path, rev = create_local_git_repo(
            files={'conanfile.py': self.conanfile.format(revision_mode="scm")})
        t = TestClient(current_folder=path)

        ref = ConanFileReference.loads("name/version@user/channel")
        t.run("export . {}".format(ref))

        meta = t.cache.package_layout(ref, short_paths=False).load_metadata()
        self.assertEqual(meta.recipe.revision, rev)

    def test_revision_mode_invalid(self):
        conanfile = self.conanfile.format(revision_mode="auto")

        t = TestClient()
        t.save({'conanfile.py': conanfile})
        ref = ConanFileReference.loads("name/version@user/channel")
        t.run("export . {}".format(ref), assert_error=True)
        self.assertIn("ERROR: Revision mode should be one of 'hash' (default) or 'scm'", t.out)

    def test_export_no_params(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
                        from conans import ConanFile

                        class MyPkg(ConanFile):
                            name = "lib"
                            version = "1.0"
                        """)
        client.save({"conanfile.py": conanfile})
        client.run('export .')
        client.cache.package_layout(ConanFileReference.loads("lib/1.0@")).export()
        self.assertIn("lib/1.0: A new conanfile.py version was exported", client.out)

        # Do it twice
        client.run('export . ')
        self.assertIn("lib/1.0: The stored package has not changed", client.out)

    def export_with_name_and_version_test(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
                from conans import ConanFile

                class MyPkg(ConanFile):
                    pass
                """)
        client.save({"conanfile.py": conanfile})

        client.run('export . lib/1.0@')
        self.assertIn("lib/1.0: A new conanfile.py version was exported", client.out)

    def export_with_only_user_channel_test(self):
        """This should be the recommended way and only from Conan 2.0"""
        client = TestClient()
        conanfile = textwrap.dedent("""
                from conans import ConanFile

                class MyPkg(ConanFile):
                    name = "lib"
                    version = "1.0"
                """)
        client.save({"conanfile.py": conanfile})

        client.run('export . @user/channel')
        self.assertIn("lib/1.0@user/channel: A new conanfile.py version was exported", client.out)



'''

This module handles user authentication and permissions
for read or write a conans or package.

This only reads from file the users and permissions.

Replace this module with other that keeps the interface or super class.

'''

from abc import ABCMeta, abstractmethod

import six

from conans.errors import AuthenticationException, ForbiddenException, InternalErrorException
from conans.model.ref import ConanFileReference


#  ############################################
#  ############ ABSTRACT CLASSES ##############
#  ############################################

@six.add_metaclass(ABCMeta)
class Authorizer(object):
    """
    Handles the access permissions to conans and packages
    """

    @abstractmethod
    def check_read_conan(self, username, ref):
        """
        username: User that request to read the conans
        ref: ConanFileReference
        """
        raise NotImplemented()

    @abstractmethod
    def check_write_conan(self, username, ref):
        """
        username: User that request to write the conans
        ref: ConanFileReference
        """
        raise NotImplemented()

    @abstractmethod
    def check_read_package(self, username, pref):
        """
        username: User that request to read the package
        pref: PackageReference
        """
        raise NotImplemented()

    @abstractmethod
    def check_write_package(self, username, pref):
        """
        username: User that request to write the package
        pref: PackageReference
        """
        raise NotImplemented()


@six.add_metaclass(ABCMeta)
class Authenticator(object):
    """
    Handles the user authentication
    """
    @abstractmethod
    def valid_user(self, username, plain_password):
        """
        username: User that request to read the conans
        plain_password:
        """
        raise NotImplemented()

#  ########################################################
#  ############ BASIC IMPLEMENTATION CLASSES ##############
#  ########################################################


class BasicAuthenticator(Authenticator):
    """
    Handles the user authentication from a dict of plain users and passwords.
    users is {username: plain-text-passwd}
    """

    def __init__(self, users):
        self.users = users

    def valid_user(self, username, plain_password):
        """
        username: User that request to read the conans
        plain_password:
        return: True if match False if don't
        """
        return username in self.users and self.users[username] == plain_password


class BasicAuthorizer(Authorizer):
    """
    Reads permissions from the config file (server.cfg)
    """

    def __init__(self, read_permissions, write_permissions):
        """List of tuples with refs and users:

        [(ref, "user, user, user"),
         (ref, "user3, user, user")] """

        self.read_permissions = read_permissions
        self.write_permissions = write_permissions

    def check_read_conan(self, username, ref):
        """
        username: User that request to read the conans
        ref: ConanFileReference
        """
        if ref.user == username:
            return

        self._check_any_rule_ok(username, self.read_permissions, ref)

    def check_write_conan(self, username, ref):
        """
        username: User that request to write the conans
        ref: ConanFileReference
        """
        if ref.user == username:
            return True

        self._check_any_rule_ok(username, self.write_permissions, ref)

    def check_delete_conan(self, username, ref):
        """
        username: User that request to write the conans
        ref: ConanFileReference
        """
        self.check_write_conan(username, ref)

    def check_read_package(self, username, pref):
        """
        username: User that request to read the package
        pref: PackageReference
        """
        self.check_read_conan(username, pref.ref)

    def check_write_package(self, username, pref):
        """
        username: User that request to write the package
        pref: PackageReference
        """
        self.check_write_conan(username, pref.ref)

    def check_delete_package(self, username, pref):
        """
        username: User that request to write the package
        pref: PackageReference
        """
        self.check_write_package(username, pref)

    def _check_any_rule_ok(self, username, rules, *args, **kwargs):
        for rule in rules:
            # raises if don't
            ret = self._check_rule_ok(username, rule, *args, **kwargs)
            if ret:  # A rule is applied ok, if not apply keep looking
                return True
        if username:
            raise ForbiddenException("Permission denied")
        else:
            raise AuthenticationException()

    def _check_rule_ok(self, username, rule, ref):
        """Checks if a rule specified in config file applies to current conans
        reference and current user"""
        try:
            rule_ref = ConanFileReference.loads(rule[0])
        except Exception:
            # TODO: Log error
            raise InternalErrorException("Invalid server configuration. "
                                         "Contact the administrator.")
        authorized_users = [_.strip() for _ in rule[1].split(",")]
        if len(authorized_users) < 1:
            raise InternalErrorException("Invalid server configuration. "
                                         "Contact the administrator.")

        # Check if rule apply ref
        if self._check_ref_apply_for_rule(rule_ref, ref):
            if authorized_users[0] == "*" or username in authorized_users:
                return True  # Ok, applies and match username
            else:
                if username:
                    if authorized_users[0] == "?":
                        return True  # Ok, applies and match any authenticated username
                    else:
                        raise ForbiddenException("Permission denied")
                else:
                    raise AuthenticationException()

        return False

    def _check_ref_apply_for_rule(self, rule_ref, ref):
        """Checks if a conans reference specified in config file applies to current conans
        reference"""
        name, version, user, channel, _ = rule_ref
        return not((name != "*" and name != ref.name) or
                   (version != "*" and version != ref.version) or
                   (user != "*" and user != ref.user) or
                   (channel != "*" and channel != ref.channel))



import os
import platform
import unittest

from nose.plugins.attrib import attr

from conans.client.tools import replace_in_file
from conans.test.utils.tools import TestClient


class CMakeGeneratorTest(unittest.TestCase):

    def test_no_check_compiler(self):
        # https://github.com/conan-io/conan/issues/4268
        file_content = '''from conans import ConanFile, CMake

class ConanFileToolsTest(ConanFile):
    generators = "cmake"

    def build(self):
        cmake = CMake(self)
        cmake.configure()
    '''

        cmakelists = '''cmake_minimum_required(VERSION 2.8)
PROJECT(conanzlib LANGUAGES NONE)
set(CONAN_DISABLE_CHECK_COMPILER TRUE)

include(conanbuildinfo.cmake)
CONAN_BASIC_SETUP()
'''
        client = TestClient()
        client.save({"conanfile.py": file_content,
                     "CMakeLists.txt": cmakelists})

        client.run('install .')
        client.run('build .')

    @attr("slow")
    @unittest.skipUnless(platform.system() == "Windows", "Requires MSBuild")
    def skip_check_if_toolset_test(self):
        file_content = '''from conans import ConanFile, CMake

class ConanFileToolsTest(ConanFile):
    generators = "cmake"
    exports_sources = "CMakeLists.txt"
    settings = "os", "arch", "compiler"

    def build(self):
        cmake = CMake(self)
        cmake.configure()
        cmake.build()
    '''
        client = TestClient()
        cmakelists = '''
set(CMAKE_CXX_COMPILER_WORKS 1)
set(CMAKE_CXX_ABI_COMPILED 1)
PROJECT(Hello)
cmake_minimum_required(VERSION 2.8)
include("${CMAKE_BINARY_DIR}/conanbuildinfo.cmake")
CONAN_BASIC_SETUP()
'''
        client.save({"conanfile.py": file_content,
                     "CMakeLists.txt": cmakelists})
        client.run("create . lib/1.0@user/channel -s compiler='Visual Studio' -s compiler.toolset=v140")
        self.assertIn("Conan: Skipping compiler check: Declared 'compiler.toolset'", client.out)


    @attr('slow')
    def no_output_test(self):
        client = TestClient()
        client.run("new Test/1.0 --sources")
        cmakelists_path = os.path.join(client.current_folder, "src", "CMakeLists.txt")

        # Test output works as expected
        client.run("install .")
        # No need to do a full create, the build --configure is good
        client.run("build . --configure")
        self.assertIn("Conan: Using cmake global configuration", client.out)
        self.assertIn("Conan: Adjusting default RPATHs Conan policies", client.out)
        self.assertIn("Conan: Adjusting language standard", client.out)

        # Silence output
        replace_in_file(cmakelists_path,
                        "conan_basic_setup()",
                        "set(CONAN_CMAKE_SILENT_OUTPUT True)\nconan_basic_setup()",
                        output=client.out)
        client.run("build . --configure")
        self.assertNotIn("Conan: Using cmake global configuration", client.out)
        self.assertNotIn("Conan: Adjusting default RPATHs Conan policies", client.out)
        self.assertNotIn("Conan: Adjusting language standard", client.out)

        # Use TARGETS
        replace_in_file(cmakelists_path, "conan_basic_setup()", "conan_basic_setup(TARGETS)",
                        output=client.out)
        client.run("build . --configure")
        self.assertNotIn("Conan: Using cmake targets configuration", client.out)
        self.assertNotIn("Conan: Adjusting default RPATHs Conan policies", client.out)
        self.assertNotIn("Conan: Adjusting language standard", client.out)
